<?php


namespace Service;


use Doctrine\ORM\EntityManager;
use Exception;
use Service\Integration\IntegrationSM;
use Teknisa\Libs\Util\Masks\Picture;
use Teknisa\Model\Teknisa\Entities\CpCall;
use Teknisa\Model\Teknisa\Entities\Orcasolicita;
use Teknisa\Model\Teknisa\Entities\Parcsoliserv;
use Teknisa\Model\Teknisa\Entities\Tarefa;
use Util\Instance\Repository;
use Util\InstanceProvider;
use Util\Utilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Zeedhi\Framework\ORM\DateTime;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;
use Util\Messages;

class Soliserv
{
    const ACTIVE = 'A';
    const IS_ISSUE = 'Y';
    const IS_TASK = 'Y';
    const FIRST_COMMENTS = 'Abertura de issue.';
    /**
     * @var EntityManager
     */
    public $entityManager;
    /**
     * @var Utilities
     */
    public $environment;

    protected $filial = '9999';

    protected $cdprocessoRastrea = array('0000010702', '0000010703', '0000010708', '0000010709');
    /**
     * @var Teknisa\NewCode
     */
    private $newCodeService;
    /**
     * @var int|mixed
     */
    private $nrorg;

    public $basicIntranetFileDir = '';

    public function __construct()
    {
        $this->entityManager  = InstanceProvider::getEntityManager();
        $this->environment    = InstanceProvider::getEnvironment();

        $this->nrorg = $this->environment->getCurrentOrganizationId();
    }

    public function findBudgetPlots($issueId, $namePlots)
    {
        return InstanceProvider::getRepository()->getBudget()
            ->findIssueBudgetPlots($issueId, $namePlots);
    }

    public function findBudget($SoliId, $nameProposal, $nameDiscount, $namePercent)
    {
        return InstanceProvider::getRepository()->getLocalSoliserv()
            ->findBudget($SoliId, $nameProposal, $nameDiscount, $namePercent);
    }

    public function validateBudget($nrsoliserv)
    {
        $return = null;
        $validate = InstanceProvider::getRepository()->getLocalSoliserv()->findValidateBudget($nrsoliserv);
        $cdfasetramBudget = Utilities::getCdfaseTramOutGenerateProposal();
        if (!empty($validate[0]['CDFASETRAM']) && $validate[0]['CDFASETRAM'] === $cdfasetramBudget) {
            $return = 'CLI_ORC';
        } else if (!empty($validate[0]['NRORCAMENTO'])) {
            $return = 'ACE_ORC';
        }
        return $return;
    }

    public function findMaintenanceCost($nrsoliserv)
    {
        $budgetRepository = InstanceProvider::getRepository()->getLocalSoliserv();
        $organizationId = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $maintenance = $budgetRepository->MaintenanceCost($organizationId);
        $result = array();
        if ($maintenance) {
            $result['MAINTENANCE_PERCENTAGE'] = $maintenance;
            $Totalvalue = $budgetRepository->findManutSoliservBudget($nrsoliserv);
            $finalPrice = round($Totalvalue, 2);
            $maintenancePrice = round($finalPrice * $maintenance / 100, 2);
            $maintenancePrice = number_format($maintenancePrice, 2, '.', '');

            $maintenanceDiscounts = $budgetRepository->findManutSoliservDiscount($nrsoliserv);
            if ($maintenanceDiscounts && $maintenanceDiscounts[0]) {
                $result['VALUE_DISCOUNT'] = $maintenanceDiscounts[0]['MAINTENANCE_VALUE'];
                $result['PERCENTAGE_DISCOUNT'] =  $maintenanceDiscounts[0]['MAINTENANCE_PERCENT'];
                $result['TOTAL_VALUE'] = $maintenancePrice - $maintenanceDiscounts[0]['MAINTENANCE_VALUE'];
            }

            $result['MANUT_VALUE'] = $maintenancePrice;
        } else {
            $result['MANUT_VALUE'] = 0;
            $result['TOTAL_VALUE'] = 0;
            $result['PERCENTAGE_DISCOUNT'] = 0;
            $result['VALUE_DISCOUNT'] = 0;
            $result['MAINTENANCE_PERCENTAGE'] = 2;
        }
        $result['VALIDATE_ERROR']  = $this->validateBudget($nrsoliserv);
        $result['VALIDATE_EDIT']  = !$result['VALIDATE_ERROR'];
        return $result;
    }

    public function findSoliBudgetPlots($issueId, $namePlots)
    {
        return InstanceProvider::getRepository()->getLocalSoliserv()
            ->findSoliBudgetPlots($issueId, $namePlots);
    }

    public function soliGenerateProposal($nrsoliserv)
    {

        /** @var \Teknisa\Model\Teknisa\Entities\Soliserv $soliserv */
        $soliserv = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliserv));
        $cdfasetram = $soliserv->getCdfasetram();
        $cdfasetramBudget = Utilities::getCdfaseTramInGenerateProposal();
        $cdfastramdes = Utilities::getCdfaseTramOutGenerateProposal();
        $clienteTeknisa = Utilities::getCdClienteBase() === $soliserv->getCdcliente();
        $propostaId = null;
        if ($cdfasetram !== $cdfasetramBudget) {
            throw new \Exception("Só é possivel criar Proposta no SM na fase $cdfasetramBudget");
        }
        if (!$clienteTeknisa) {
            $proposal = InstanceProvider::getRepository()->getLocalSoliserv()->findProposaByNrsoliserv($nrsoliserv);
            if (!empty($proposal)) {
                IntegrationSM::integrationSMCancelSoliserv($nrsoliserv, $soliserv->getNrorg());
            }
            IntegrationSM::integrationSMCreateProposalByNrsoliserv($nrsoliserv);
        }
    }

    public function soliGenerateProposalValidate($nrsoliserv)
    {
        /** @var \Teknisa\Model\Teknisa\Entities\Soliserv $soliserv */
        $soliserv = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliserv));
        $validate = 'OK';
        $type = 'OK';
        $cdfaseInProposal = Utilities::setSelectMultiInQuery(Utilities::getCdfaseTramInGenerateProposal(), false, "','");
        if (!in_array($soliserv->getCdfasetram(), $cdfaseInProposal)) {
            $validate = 'FASE_ERROR';
            $type = 'ERROR';
        } else {
            $proposal = InstanceProvider::getRepository()->getLocalSoliserv()->findProposaByNrsoliserv($nrsoliserv);
            if (!empty($proposal)) {
                $validate = 'PRO_CRE_ALE';
                $type = 'ALERT';
            }
        }
        return ['validate' => $validate, 'type' => $type];
    }

    public function soliSMBudgetValidate($nrsoliserv)
    {
        $proposal = InstanceProvider::getRepository()->getLocalSoliserv()->findProposaByNrsoliserv($nrsoliserv);
        if (empty($proposal)) {
            throw new \Exception('Orçamento não encontrado no SM!');
        }

        return ['validate' => 'OK', 'type' => 'OK'];
    }

    public function soliSpeCliValidate($nrsoliserv)
    {
        $proposal = InstanceProvider::getService()->getAttachment()->getPageRequisiteValidate($nrsoliserv);
        if (empty($proposal)) {
            $this->mergeFileSpecificationTaskS($nrsoliserv);
        }

        return ['validate' => 'OK', 'type' => 'OK'];
    }

    public function soliBudgetPlotsSave($rows, $nrsoliserv)
    {

        $connection = InstanceProvider::getEntityManager()->getConnection();
        try {
            $connection->beginTransaction();

            $instanceRepository = InstanceProvider::getRepository();
            $entityManager = InstanceProvider::getEntityManager();
            $factoryBudgetPlots = InstanceProvider::getFactory()->getSoliserv();
            /** @var \Teknisa\Model\Teknisa\Repositories\Parcsoliserv $repositoryBudgetPlots */
            $repositoryBudgetPlots = $instanceRepository->getParcsoliserv();

            /** @var \Teknisa\Model\Teknisa\Entities\Soliserv $soliserv */
            $soliserv = $instanceRepository->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliserv));

            $lastParcel = 0;

            foreach ($rows as $row) {
                $orderPlots = intval($row['ORDER_PLOTS_NUM']);
                if ($lastParcel < $orderPlots) {
                    $lastParcel = $orderPlots;
                }
                $valuePlots = $row['VALUE_PLOTS'];
                /** @var Parcsoliserv $budgetPlots */
                $budgetPlots = $repositoryBudgetPlots->findOneBy(array('nrsoliserv' => $nrsoliserv, 'nrparcsoli' => $orderPlots));
                if ($budgetPlots) {
                    $budgetPlots = $factoryBudgetPlots->updateParcsoliserv($budgetPlots, $orderPlots, $valuePlots);
                } else {
                    $budgetPlots = $factoryBudgetPlots->createParcsoliserv($soliserv, $orderPlots, $valuePlots);
                }
                $entityManager->persist($budgetPlots);
            }
            $entityManager->flush();
            if ($lastParcel) {
                $budgetPlotsIdToDelete = $instanceRepository->getLocalSoliserv()->findBudgetPlotsToDelete($nrsoliserv, $lastParcel);
                $budgetPlotsToDelete = $repositoryBudgetPlots->findBy(array('nrsoliserv' => $nrsoliserv, 'nrparcsoli' => $budgetPlotsIdToDelete));
                foreach ($budgetPlotsToDelete as $parcelToDelete) {
                    $entityManager->remove($parcelToDelete);
                }
                if (!empty($budgetPlotsToDelete)) {
                    $entityManager->flush();
                }
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }


    public function SoliBudgetSave($data)
    {
        if (isset($data[0]['changed']) && $data[0]['changed']) {
            $rows = [];
            foreach ($data as $datum) {
                if (!empty($datum['NRSOLISERV'])) {
                    unset($datum['groupHeaders']);
                    $rows[] = $datum;
                }
            }
            $this->soliBudgetsaveItens($rows);
        }
    }

    public function soliBudgetRowSave($nrsoliserv, $budgetItemId, $proposalQuantity, $proposalValue, $proposalQuantityEstimada)
    {

        $entityManager = InstanceProvider::getEntityManager();
        $instanceRepository = InstanceProvider::getRepository();
        $instanceFactory = InstanceProvider::getFactory();
        /** @var \Teknisa\Model\Teknisa\Entities\Soliserv $soliserv */
        $soliserv = $instanceRepository->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliserv));
        if ($budgetItemId == '999999XXXX') {
            $proposalQuantity = abs($proposalQuantity);
            $proposalValue = abs($proposalValue);
            if ($proposalQuantity > 100) {
                $proposalQuantity = 100;
            }
            $soliserv->setVrpedesorcam($proposalQuantity);
            $soliserv->setVrdescorcam($proposalValue);
            $entityManager->persist($soliserv);
            $entityManager->flush();
        } else {
            $filterBudgetItem = array(
                'nrsoliserv' => $nrsoliserv,
                'cdfilial' => $soliserv->getCdfilial(),
                'nrorg' => $soliserv->getNrorg(),
                'cditorcaprod' => $budgetItemId,
            );
            /** @var Orcasolicita $orcasolicita  */
            $orcasolicita  = $instanceRepository->getOrcasolicita()->findOneBy($filterBudgetItem);
            $orcasolicita = $instanceFactory->getSoliserv()->saveOrcasolicita($orcasolicita, $soliserv, $budgetItemId);
            if ($proposalQuantity || $proposalQuantityEstimada) {
                $orcasolicita->setQtitorcsolic($proposalQuantity);
                $orcasolicita->setVrorcadosoli($proposalValue);
                $orcasolicita = $instanceFactory->getSoliserv()->setOrcasolicitaEstimatValues($orcasolicita);
                $entityManager->persist($orcasolicita);
                $entityManager->flush();
            }
        }
    }

    public function SoliBudgetsaveItens($rows)
    {

        $entityManager = InstanceProvider::getEntityManager();
        $instanceRepository = InstanceProvider::getRepository();
        $connection = $entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $budgetId = null;
            $TotalProposalValue = 0;
            foreach ($rows as $row) {
                $nrsoliserv = $row['NRSOLISERV'];
                $budgetItemId = $row['BUDGET_ITEM_ID'];
                $proposalQuantity = $row['PROPOSAL_QUANTITY'];
                $proposalValue = $row['PROPOSAL_VALUE'];
                $proposalQuantityEstimada = $row['PROPOSAL_QUANTITY_ESTIMAT'];
                $TotalProposalValue += $proposalValue;
                $this->SoliBudgetRowSave(
                    $nrsoliserv,
                    $budgetItemId,
                    $proposalQuantity,
                    $proposalValue,
                    $proposalQuantityEstimada
                );
            }

            $repositoryBudgetPlots = $instanceRepository->getParcsoliserv();
            /** @var \Teknisa\Model\Teknisa\Entities\Soliserv $soliserv */
            $soliserv = $instanceRepository->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliserv));
            $budgetPlotsToDelete = $repositoryBudgetPlots->findBy(array('nrsoliserv' => $nrsoliserv));
            foreach ($budgetPlotsToDelete as $parcelToDelete) {
                $entityManager->remove($parcelToDelete);
            }
            if (!empty($budgetPlotsToDelete)) {
                $entityManager->flush();
            }

            $budgetPlotInitial = InstanceProvider::getFactory()->getSoliserv()->createParcsoliserv($soliserv, 1, $TotalProposalValue);
            $entityManager->persist($budgetPlotInitial);
            $entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function saveMaintenanceCost($row)
    {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        try {
            $connection->beginTransaction();
            if ($row['Soli_ID']) {
                /** @var \Teknisa\Model\Teknisa\Entities\Soliserv $soliserv */
                $soliserv = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $row['Soli_ID']));

                $valueDiscount = str_replace(",", ".", $row['VALUE_DISCOUNT']);
                $valueManut = str_replace(",", ".", $row['MANUT_VALUE']);
                $valueDiscount = floatval(preg_replace('/[^\d\.]+/', '', $valueDiscount));
                $valuePercent = str_replace(",", ".", $row['PERCENTAGE_DISCOUNT']);
                if ($valueManut < $valueDiscount) {
                    $valueDiscount = $valueManut;
                }
                if ($valuePercent > 100) {
                    $valuePercent = 100;
                }
                $soliserv->setVrpedesmanut($valuePercent);
                $soliserv->setVrdescmanut($valueDiscount);
                $entityManager->persist($soliserv);
                $entityManager->flush();
                $connection->commit();
            }
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function findTarefa($nrsoliserv)
    {
        $cdassuntoEspec = Utilities::getCdassuntoExpec();
        return InstanceProvider::getRepository()->getSoliservLocal()->findTarefa($nrsoliserv, $cdassuntoEspec);
    }

    public function findAllSoliserv($dynamicFilter)
    {
        return InstanceProvider::getRepository()->getSoliservLocal()->findAllSoliserv();
    }

    public function findItemTarefa($nrtarefa, $nrsoliserv)
    {
        return InstanceProvider::getRepository()->getSoliservLocal()->findItemTarefa($nrtarefa, $nrsoliserv);
    }

    public function findAddItemTarefa()
    {
        return InstanceProvider::getRepository()->getSoliservLocal()->findAddItemTarefa();
    }

    public function findValidadeAtividateGerada($nrtarefas, $cdprocessos, $cditemtrabalhos)
    {
        $cdfilial = $this->filial;
        return InstanceProvider::getRepository()->getSoliservLocal()->findValidadeAtividateGerada($cdfilial, $nrtarefas, $cdprocessos, $cditemtrabalhos);
    }

    public function findProcessTotalized($nrtarefa)
    {
        $cdfilial = $this->filial;
        $return = InstanceProvider::getRepository()->getSoliservLocal()->findProcessTotalized($cdfilial, $nrtarefa);
        $picture = $this->getPicture();
        foreach ($return['data'] as &$item) {
            $item['ID_TREE'] = $picture->getCodigoComMascara($item['ID_TREE']);
        }
        return $return;
    }

    public function findActivityDetails($nrtarefas, $nrplativtare, $cdFilial)
    {
        $desc = InstanceProvider::getRepository()->getSoliservLocal()->findActivityDescriptionAndRespons($nrtarefas, $nrplativtare, $cdFilial);
        $return = array(
            'DESCRIPTION' => '',
            'RESPONSE' => '',
        );
        if (!empty($desc)) {
            $return['DESCRIPTION'] = Utilities::rtf2textPlain($desc[0]['TXRFATIVTARE'], true);
            $return['RESPONSE'] = Utilities::rtf2textPlain($desc[0]['TXREATIVTARE'], true);
        }
        return $return;
    }

    public function findPlanativtare($nrtarefa)
    {
        $cdfilial = $this->filial;
        $return = InstanceProvider::getRepository()->getSoliservLocal()->findPlanativtare($cdfilial, $nrtarefa);
        foreach ($return['data'] as &$datum) {
            $datum['IS_DELAYED'] = false;
            if ($datum['DTFIPATIVTAR']) {
                if ($datum['VALIDATE_DATE'] < $datum['VALIDATE_NOW']) {
                    $datum['IS_DELAYED'] = true;
                }
            }
        }
        return $return;
    }

    public function generateActivitiesSoliserv($nrtarefa, $cdarvprocess, $nrsoliserv, $processos, $itemtrabalhos)
    {
        $connection           = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $this->newCodeService = InstanceProvider::getService()->getTeknisaNewCode();
            $soliservLocal = InstanceProvider::getRepository()->getSoliservLocal();
            $factoryPlativtare = InstanceProvider::getFactory()->getPlanativtare();
            $factoryDescplativ = InstanceProvider::getFactory()->getDescplativ();
            $factoryPlAtvRelItTr = InstanceProvider::getFactory()->getPlAtvRelItTr();
            $currentCdoperador = $this->environment->getCurrentCdoperador();
            $dtincativtar = new \DateTime();
            $cdoperincl = $currentCdoperador;
            $nrcplxitativ = 0;
            foreach ($processos as $process) {
                $cdprocesso = $process["CDPROCESSO"];
                list($nrplativtare, $cdfilial, $process, $cdoperresp, $activity) =
                    $this->insetBeforeForItem(
                        $process,
                        $itemtrabalhos,
                        $factoryPlativtare,
                        $nrtarefa,
                        $cdprocesso,
                        $dtincativtar,
                        $cdoperincl,
                        $nrcplxitativ
                    );

                foreach ($itemtrabalhos as $itemtrabalho) {

                    $valorComplexidade = $this->getValorComplexidade($process, $itemtrabalho);

                    if ($process['IDGERATVCONS'] === 'S') {
                        $activity->setNrcplxitativ($activity->getNrcplxitativ() + $valorComplexidade);
                        $this->entityManager->persist($activity);
                        $this->entityManager->flush();
                    } else {
                        list($nrplativtare, $cdfilial, $process, $cdoperresp, $activity, $qttemplaitar)
                            = $this->insertPLanAtiv(
                                $itemtrabalho,
                                $process,
                                $factoryPlativtare,
                                $nrtarefa,
                                $cdprocesso,
                                $dtincativtar,
                                $cdoperincl,
                                $valorComplexidade
                            );
                        $this->entityManager->persist($activity);
                        $this->entityManager->flush();

                        list($description, $itemtrabalho) = $this->getDescription(
                            $itemtrabalho,
                            $soliservLocal,
                            $cdarvprocess,
                            $nrsoliserv,
                            $cdfilial,
                            $nrtarefa
                        );
                        if (!empty($description)) {
                            $descplativ = $factoryDescplativ->create($cdfilial, $nrtarefa, $nrplativtare, $description, $this->nrorg);
                            $this->entityManager->persist($descplativ);
                        }
                    }
                    $nrsequittare = $itemtrabalho['NRSEQUITTARE'];
                    $plAtvRelItTr = $factoryPlAtvRelItTr->create($cdfilial, $nrtarefa, $nrplativtare, $nrsequittare);
                    $this->entityManager->persist($plAtvRelItTr);
                    $this->entityManager->flush();
                }

                $idgeratvcons = $process['IDGERATVCONS'];
                if ($idgeratvcons === 'S') {
                    $this->entityManager->refresh($activity);
                    list($vrprodmedes, $pesoCargo, $vrpesoativgrc, $qttempminativ, $qttemplaitar) =
                        $this->setActivity($activity, $process, $soliservLocal, $cdoperresp);
                    $this->entityManager->persist($activity);
                }

                $this->setsRastreabilidade(
                    $cdprocesso,
                    $soliservLocal,
                    $cdfilial,
                    $nrtarefa,
                    $activity,
                    $process,
                    $cdoperresp
                );
                $this->entityManager->flush();
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            try {
                $connection->rollBack();
                throw $e;
            } catch (\Exception $e2) {
            }
            throw $e;
        }
        return [1];
    }

    public function findTarefaAllResponsavel($nrsoliserv)
    {
        return InstanceProvider::getRepository()->getSoliservLocal()->findTarefaAllResponsavel($nrsoliserv);
    }

    public function findTarefaAssunto()
    {
        return InstanceProvider::getRepository()->getSoliservLocal()->findTarefaAssunto();
    }

    public function getParFilialGep($filial = null)
    {
        if (empty($filial)) {
            $filial = $this->filial;
        }
        return InstanceProvider::getRepository()->getSoliservLocal()->getParFilialGep($filial);
    }

    public function getPicture()
    {
        return new Picture($this->environment->getCurrentOrganizationId(), 'CDPICTPROCES');
    }

    public function findProcessGeneration()
    {
        $parFilialGep = $this->getParFilialGep();
        $cdsetor = $parFilialGep['CDSETORPROC'];
        $cdnvprocesso = $parFilialGep['CDNVPESQPROC'];
        $return = InstanceProvider::getRepository()->getSoliservLocal()->findProcessGeneration($cdsetor, $cdnvprocesso);
        $picture = $this->getPicture();
        foreach ($return['data'] as &$item) {
            $item['ID_TREE'] = $picture->getCodigoComMascara($item['ID_TREE']);
        }
        return $return;
    }

    public function findActivitiesGeneration($cdarvprocess)
    {
        $picture = $this->getPicture();
        $cdnvprocesso = $picture->getNumeroDeNiveis();
        if (empty($cdarvprocess)) {
            $cdarvprocess = 9999999;
        }
        return InstanceProvider::getRepository()->getSoliservLocal()->findActivitiesGeneration($cdarvprocess, $cdnvprocesso);
    }

    public function findDefaultTaskSoli($nrsoliserv)
    {
        $currentCdoperador = $this->environment->getCurrentCdoperador();
        return InstanceProvider::getRepository()->getSoliservLocal()->findDefaultTaskSoli($nrsoliserv, $currentCdoperador);
    }

    public function validateFornwardSoli($nrsoliserv)
    {
        $validate = true;
        $validateFornwardSoli = InstanceProvider::getRepository()->getSoliservLocal()->validateFornwardSoli($nrsoliserv);
        if ($validateFornwardSoli > 0) {
            $validate = false;
        }
        return $validate;
    }

    public function saveTarefaSoli($nrsoliserv, $cdassunto, $cdoperresp, $dsrefetarefa, $idorigtare, $nrpriotarefa)
    {
        $currentCdoperador = $this->environment->getCurrentCdoperador();
        $tarefa = InstanceProvider::getFactory()->getTarefa()->createTarefa($nrsoliserv, $cdassunto, $cdoperresp, $dsrefetarefa, $idorigtare, $nrpriotarefa, $currentCdoperador);
        $this->entityManager->persist($tarefa);
        $this->entityManager->flush();
    }

    public function editTarefaSoli($nrtarefa, $cdoperresp, $dsrefetarefa, $nrpriotarefa)
    {
        $tarefa = InstanceProvider::getRepository()->getTarefa()->findOneBy(array('nrtarefa' => $nrtarefa));
        $tarefa = InstanceProvider::getFactory()->getTarefa()->editTarefa($tarefa, $cdoperresp, $dsrefetarefa, $nrpriotarefa);
        $this->entityManager->persist($tarefa);
        $this->entityManager->flush();
    }

    public function saveItemTarefaSoli($cditemtrab, $nrtarefa, $nrprioittare, $nrcplximplem, $nrcplxitemtb)
    {
        $qryVerificaItemTrab = InstanceProvider::getRepository()->getLocalSoliserv()->qryVerificaItemTrab($cditemtrab, $nrtarefa);
        if ($qryVerificaItemTrab[0]['QTD'] > 0) {
            throw new \Exception(Messages::ITEM_TRAB_EXITS);
        } else {
            $currentCdoperador = $this->environment->getCurrentCdoperador();
            /** @var Tarefa $tarefa */
            $tarefa = InstanceProvider::getRepository()->getTarefa()->findOneBy(array('nrtarefa' => $nrtarefa));
            $cdfilial = $tarefa->getCdfilial();
            $dsrefeittare = $tarefa->getDsrefetarefa();
            $nrsequittare = InstanceProvider::getRepository()->getSoliservLocal()->getNrsequittare($nrtarefa, $cdfilial);
            $parFilialGep = $this->getParFilialGep($cdfilial);
            $vrpesoaltcpx = $parFilialGep['VRPESOALTCPX'];
            $itemtarefa = InstanceProvider::getFactory()->getItemTarefa()->createItemTarefa(
                $nrtarefa,
                $cditemtrab,
                $cdfilial,
                $nrsequittare,
                $dsrefeittare,
                $nrprioittare,
                $nrcplximplem,
                $nrcplxitemtb,
                $currentCdoperador,
                $vrpesoaltcpx
            );
            $this->entityManager->persist($itemtarefa);
            $this->entityManager->flush();
        }
    }

    public function editItemTarefaSoli($nrsequittare, $nrtarefa, $nrprioittare, $nrcplximplem, $nrcplxitemtb)
    {
        $currentCdoperador = $this->environment->getCurrentCdoperador();
        /** @var Tarefa $tarefa */
        $tarefa = InstanceProvider::getRepository()->getTarefa()->findOneBy(array('nrtarefa' => $nrtarefa));
        /** @var Tarefa $tarefa */
        $itemTarefa = InstanceProvider::getRepository()->getItemTarefa()->findOneBy(array('nrtarefa' => $nrtarefa, 'nrsequittare' => $nrsequittare));
        $cdfilial = $tarefa->getCdfilial();
        $dsrefeittare = $tarefa->getDsrefetarefa();
        $parFilialGep = $this->getParFilialGep($cdfilial);
        $vrpesoaltcpx = $parFilialGep['VRPESOALTCPX'];
        $itemtarefa = InstanceProvider::getFactory()->getItemTarefa()->editItemTarefa(
            $itemTarefa,
            $cdfilial,
            $nrsequittare,
            $dsrefeittare,
            $nrprioittare,
            $nrcplximplem,
            $nrcplxitemtb,
            $currentCdoperador,
            $vrpesoaltcpx
        );
        $this->entityManager->persist($itemtarefa);
        $this->entityManager->flush();
    }

    public function deleteTarefaSoli($nrtarefa)
    {
        $tarefa = InstanceProvider::getRepository()->getTarefa()->findOneBy(array('nrtarefa' => $nrtarefa));
        $this->entityManager->remove($tarefa);
        $this->entityManager->flush();
    }

    public function deleteItemTarefasSoli($nrtarefa, $nrsequittare)
    {
        $itemTarefa = InstanceProvider::getRepository()->getItemTarefa()->findOneBy(array('nrtarefa' => $nrtarefa, 'nrsequittare' => $nrsequittare));
        $this->entityManager->remove($itemTarefa);
        $this->entityManager->flush();
    }

    public function downloadFileSpecificationTaskS($nrsoliserv, $cditemtrab)
    {

        InstanceProvider::getService()->getAttachment()->downloadFileSpecificationTaskS($nrsoliserv, $cditemtrab);
    }
    public function getLinkDownloadBudgetSmSoliserv($nrsoliserv)
    {
        $nrorg = Utilities::getEnvironment()->getCurrentOrganizationId();
        $linkDownloadBudgetSm =  IntegrationSM::linkDownloadBudgetSmSoliserv($nrsoliserv, $nrorg);
        \Util\Instance\Service::getAttachment()->downloadBudgetSm($linkDownloadBudgetSm);

        $zhFileUploader = InstanceProvider::getFileUploaderServe();
        $zhFileUploader->deleteFile($linkDownloadBudgetSm);
    }

    public function insertFileSpecificationTaskS($nrsoliserv, $cditemtrab, $document)
    {
        InstanceProvider::getService()->getAttachment()->insertFileSpecificationTaskS($nrsoliserv, $cditemtrab, $document);
    }

    public function mergeFileSpecificationTaskS($nrsoliserv)
    {
        InstanceProvider::getService()->getINTRANETIntegration()->mergeFileSpecificationTaskS($nrsoliserv);
    }

    public function editSpecificationSoliserv($nrsoliserv, $cditemtrab)
    {
        $file = InstanceProvider::getService()->getINTRANETIntegration()->getFileNameDocSpecificationTaskS($nrsoliserv, $cditemtrab);
        return InstanceProvider::getService()->getIntegrationGoogleDrive()->uploadDoc($file);
    }

    public function saveSpecificationSoliserv($nrsoliserv, $cditemtrab, $idGoogle)
    {
        $file = InstanceProvider::getService()->getINTRANETIntegration()->getFileNameDocSpecificationTaskS($nrsoliserv, $cditemtrab);
        InstanceProvider::getService()->getIntegrationGoogleDrive()->saveDoc($file, $idGoogle);
    }

    public function cancelSpecificationSoliserv($idGoogle)
    {
        InstanceProvider::getService()->getIntegrationGoogleDrive()->deleteFileDrive($idGoogle);
    }

    public function editGridActivityS(
        $cdfilial,
        $nrtarefa,
        $nrplativtare,
        $nrprioativtr,
        $cdoperresp,
        $qthrinfativtar
    ) {
        /** @var \Teknisa\Model\Teknisa\Entities\Planativtare $planativtare */
        $planativtare = InstanceProvider::getRepository()->getPlanativtare()->findOneBy(array('nrtarefa' => $nrtarefa, 'cdfilial' => $cdfilial, 'nrplativtare' => $nrplativtare));
        $planativtare->setNrprioativtr($nrprioativtr);
        $planativtare->setcdoperresp($cdoperresp);
        $planativtare->setQthrinfativtar($qthrinfativtar);
        $this->entityManager->persist($planativtare);
        $this->entityManager->flush();
    }

    public function vdValidacaoPar($cdoperresp, $nmprocesso, $cdoperincl)
    {
        $qryPermite = InstanceProvider::getRepository()->getSoliservLocal()->qryPermite($cdoperresp);
        $nmprocessoUpper = strtoupper($nmprocesso);
        if (empty($qryPermite)) {
            if (in_array('PARES', preg_split('/[ !,.?]+/', $nmprocessoUpper)) == true) {
                throw new \Exception(Messages::EXCL_INV);
            }
            if (in_array('ERRO DO PATCH', preg_split('/[ !,.?]+/', $nmprocessoUpper)) == true) {
                throw new \Exception(Messages::EXCL_PATCH);
            }
            if ($cdoperincl != $cdoperresp) {
                throw new \Exception(Messages::OP_INCL);
            }
        } elseif ($cdoperincl != $cdoperresp) {
            throw new \Exception(Messages::OP_INCL);
        }
    }

    public function vdAtivCSA($nrtarefa, $nrplativtare)
    {
        $qryVdAtivCSA = InstanceProvider::getRepository()->getSoliservLocal()->qryVdAtivCSA($nrtarefa, $nrplativtare);
        if (!empty($qryVdAtivCSA)) {
            throw new \Exception(Messages::N_EXC_ATIV_CSA);
        }
    }

    public function verificaAproativtare($nrtarefa, $nrplativtare)
    {
        $qryVerificaAproativtare = InstanceProvider::getRepository()->getSoliservLocal()->qryVerificaAproativtare($nrtarefa, $nrplativtare);
        if (!empty($qryVerificaAproativtare)) {
            $connection = InstanceProvider::getEntityManager()->getConnection();
            try {
                $connection->beginTransaction();
                InstanceProvider::getRepository()->getSoliservLocal()->qryDelAproativtare($nrtarefa, $nrplativtare);
                $connection->commit();
            } catch (\Exception $e) {
                try {
                    $connection->rollBack();
                    throw $e;
                } catch (\Exception $e2) {
                }
                throw $e;
            }
        }
    }

    public function verificaRespplativ($nrtarefa, $nrplativtare)
    {
        $qryVerificaRespplativ = InstanceProvider::getRepository()->getSoliservLocal()->qryVerificaRespplativ($nrtarefa, $nrplativtare);
        if (!empty($qryVerificaRespplativ)) {
            $connection = InstanceProvider::getEntityManager()->getConnection();
            try {
                $connection->beginTransaction();
                InstanceProvider::getRepository()->getSoliservLocal()->qryDelRespplativ($nrtarefa, $nrplativtare);
                $connection->commit();
            } catch (\Exception $e) {
                try {
                    $connection->rollBack();
                    throw $e;
                } catch (\Exception $e2) {
                }
                throw $e;
            }
        }
    }

    public function deleteGridActivityS($nrtarefa, $cdfilial, $nrplativtare, $idsitativtar, $cdoperresp, $nmprocesso, $cdoperincl)
    {
        if ($idsitativtar == 'P') {
            $this->vdValidacaoPar($cdoperresp, $nmprocesso, $cdoperincl);
            $this->vdAtivCSA($nrtarefa, $nrplativtare);
            $this->verificaAproativtare($nrtarefa, $nrplativtare);
            $this->verificaRespplativ($nrtarefa, $nrplativtare);
            /** @var \Teknisa\Model\Teknisa\Entities\Descplativ[] $descplativs */
            $descplativs = InstanceProvider::getRepository()->getDescplativ()->findBy(array('nrtarefa' => $nrtarefa, 'cdfilial' => $cdfilial, 'nrplativtare' => $nrplativtare));
            foreach ($descplativs as $descplativ) {
                $this->entityManager->remove($descplativ);
            }
            /** @var \Teknisa\Model\Teknisa\Entities\Platvrelittr[] $planativtare */
            $plAtvRelItTrs = InstanceProvider::getRepository()->getPlAtvRelItTr()->findBy(array('nrtarefa' => $nrtarefa, 'cdfilial' => $cdfilial, 'nrplativtare' => $nrplativtare));
            foreach ($plAtvRelItTrs as $plAtvRelItTr) {
                $this->entityManager->remove($plAtvRelItTr);
            }
            /** @var \Teknisa\Model\Teknisa\Entities\Planativtare $planativtare */
            $planativtare = InstanceProvider::getRepository()->getPlanativtare()->findOneBy(array('nrtarefa' => $nrtarefa, 'cdfilial' => $cdfilial, 'nrplativtare' => $nrplativtare));
            $this->entityManager->remove($planativtare);
            $this->entityManager->flush();
        } else {
            throw new \Exception(Messages::ACTIVY_DELETE);
        }
    }

    /**
     * @param $process
     * @param $itemtrabalho
     * @return float|mixed
     */
    public function getValorComplexidade($process, $itemtrabalho)
    {
        if ($process['IDREFECPLX'] === 'A') {
            $valorComplexidade = $itemtrabalho['NRCPLXIMPLEM'] + round($process, 2);
        } else {
            $valorComplexidade = $itemtrabalho['NRCPLXITTARE'] + round($process, 2);
        }
        return $valorComplexidade;
    }

    /**
     * @param $nrtarefa
     * @return String
     */
    public function getPrxPlanAtivTare($nrtarefa): string
    {
        $nrOrg = $this->environment->getCurrentOrganizationId();
        $cdfilial = $this->filial;
        return $this->newCodeService->getCodeByTableName('PLANATIVTARE' . $cdfilial . $nrtarefa, $nrOrg, 1, 5);
    }

    /**
     * @param $process
     * @param $itemtrabalhos
     * @param \Factory\Planativtare $factoryPlativtare
     * @param $nrtarefa
     * @param $cdprocesso
     * @param \DateTime $dtincativtar
     * @param $cdoperincl
     * @param int $nrcplxitativ
     * @return array
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function insetBeforeForItem(
        $process,
        $itemtrabalhos,
        \Factory\Planativtare $factoryPlativtare,
        $nrtarefa,
        $cdprocesso,
        \DateTime $dtincativtar,
        $cdoperincl,
        int $nrcplxitativ
    ): array {
        $cdoperresp = null;
        $activity = null;
        $nrplativtare = null;
        $cdfilial = $itemtrabalhos[0]["CDFILIAL"];
        if ($process['IDGERATVCONS'] === 'S') {
            $nrplativtare = $this->getPrxPlanAtivTare($nrtarefa);

            $cdoperresp = empty($process["CDOPERADOR"]) ? $itemtrabalhos[0]["CDOPERADOR"] : $process["CDOPERADOR"];

            /** @var \Teknisa\Model\Teknisa\Entities\Planativtare $activity */
            $activity = $factoryPlativtare->create(
                $nrplativtare,
                $cdfilial,
                $nrtarefa,
                $cdprocesso,
                $cdoperresp,
                $dtincativtar,
                $cdoperincl,
                $nrcplxitativ
            );
            $this->entityManager->persist($activity);
            $this->entityManager->flush();
            $this->entityManager->refresh($activity);
        }
        return array($nrplativtare, $cdfilial, $process, $cdoperresp, $activity);
    }

    /**
     * @param $itemtrabalho
     * @param $process
     * @param \Factory\Planativtare $factoryPlativtare
     * @param $nrtarefa
     * @param $cdprocesso
     * @param \DateTime $dtincativtar
     * @param $cdoperincl
     * @param float $valorComplexidade
     * @return array
     */
    public function insertPLanAtiv($itemtrabalho, $process, \Factory\Planativtare $factoryPlativtare, $nrtarefa, $cdprocesso, \DateTime $dtincativtar, $cdoperincl, float $valorComplexidade): array
    {
        $nrplativtare = $this->getPrxPlanAtivTare($nrtarefa);
        $cdfilial = $itemtrabalho["CDFILIAL"];
        $cdoperresp = empty($process["CDOPERADOR"]) ? $itemtrabalho["CDOPERADOR"] : $process["CDOPERADOR"];
        $activity = $factoryPlativtare->create(
            $nrplativtare,
            $cdfilial,
            $nrtarefa,
            $cdprocesso,
            $cdoperresp,
            $dtincativtar,
            $cdoperincl,
            $valorComplexidade
        );
        $activity->setCditemtrab($itemtrabalho['CDITEMTRAB']);
        $activity->setNrcplxplanat($valorComplexidade);
        $qttemplaitar = (($valorComplexidade / $process['VRPRODMEDES']) + $process['QTTEMPMINATIV']);
        $activity->setQttemplaitar($qttemplaitar);
        if ($qttemplaitar > 0) {
            $activity->setNrcplxitativ(round(($qttemplaitar * $process['VRPRODMEDES']), 2));
            $activity->setNrcplxplanat($activity->getNrcplxitativ());
        }
        return array($nrplativtare, $cdfilial, $process, $cdoperresp, $activity, $qttemplaitar);
    }

    /**
     * @param $itemtrabalho
     * @param \Repository\Soliserv $soliservLocal
     * @param $cdarvprocess
     * @param $nrsoliserv
     * @param $cdfilial
     * @param $nrtarefa
     * @return array
     */
    public function getDescription($itemtrabalho, \Repository\Soliserv $soliservLocal, $cdarvprocess, $nrsoliserv, $cdfilial, $nrtarefa): array
    {
        $description = '';
        if (!empty($itemtrabalho['TXPLANITTARE'])) {
            $description = $itemtrabalho['TXPLANITTARE'];
        } else {
            $qryVerifCor = $soliservLocal->findQryVerifCor($cdarvprocess);
            $qryVerifVerif = $soliservLocal->findQryVerifVerif($nrsoliserv, $this->filial);
            if (!empty($qryVerifCor) && !empty($qryVerifVerif)) {
                $description = $qryVerifVerif[0]['TXREATIVTARE'];
            } else {
                $descTarefa = $soliservLocal->findqryDescTarefa($cdfilial, $nrtarefa);
                if (!empty($descTarefa)) {
                    $description = $descTarefa[0]['TXTAREFA'];
                }
            }
        }
        return array($description, $itemtrabalho);
    }

    /**
     * @param $activity
     * @param $process
     * @param \Repository\Soliserv $soliservLocal
     * @param $cdoperresp
     * @return array
     */
    public function setActivity($activity, $process, \Repository\Soliserv $soliservLocal, $cdoperresp): array
    {
        $activity->setNrcplxplanat($activity->getNrcplxitativ());
        $vrprodmedes = $process['VRPRODMEDES'];
        if ($vrprodmedes > 0) {
            $pesoCargo = $soliservLocal->findqryPesoCargo($cdoperresp);
            $vrpesoativgrc = $pesoCargo[0]['VRPESOATIVGRC'];
            $qttempminativ = $process['QTTEMPMINATIV'];
            $qttemplaitar = (($activity->getNrcplxplanat() / $vrprodmedes) * $vrpesoativgrc) + $qttempminativ;
            $activity->setQttemplaitar($qttemplaitar);
            if ($qttemplaitar > 0) {
                $activity->setNrcplxitativ(round(($activity->getQttemplaitar() * $vrprodmedes), 2));
                $activity->setNrcplxplanat(round(($activity->getQttemplaitar() * $vrprodmedes), 2));
            }
        } else {
            $activity->setQttemplaitar(0);
        }
        return array($vrprodmedes, $pesoCargo, $vrpesoativgrc, $qttempminativ, $qttemplaitar);
    }

    /**
     * @param $cdprocesso
     * @param \Repository\Soliserv $soliservLocal
     * @param $cdfilial
     * @param $nrtarefa
     * @param $activity
     * @param $process
     * @param $cdoperresp
     * @throws \Doctrine\ORM\ORMException
     */
    public function setsRastreabilidade($cdprocesso, \Repository\Soliserv $soliservLocal, $cdfilial, $nrtarefa, $activity, $process, $cdoperresp): void
    {
        if (in_array($cdprocesso, $this->cdprocessoRastrea)) {
            $qryRastreabilidade = $soliservLocal->findqryRastreabilidade($cdfilial, $nrtarefa);
            if (!empty($qryRastreabilidade)) {
                foreach ($qryRastreabilidade as $rastre) {
                    $activity->setNrcplxitativ($activity->getNrcplxitativ() + ($rastre['NRCPLXITTRAB'] * $process['VRFATOCPLX']));
                    $activity->setNrcplxplanat($activity->getNrcplxitativ());
                    $this->entityManager->persist($activity);
                }
                $vrprodmedes = $process['VRPRODMEDES'];
                if ($vrprodmedes > 0) {
                    $pesoCargo = $soliservLocal->findqryPesoCargo($cdoperresp);
                    $vrpesoativgrc = $pesoCargo[0]['VRPESOATIVGRC'];
                    $qttempminativ = $process;
                    $qttemplaitar = (($activity->getNrcplxplanat() / $vrprodmedes) * $vrpesoativgrc) + $qttempminativ;
                    $activity->setQttemplaitar($qttemplaitar);
                } else {
                    $activity->setQttemplaitar(0);
                }
            }
        }
    }

    public function saveItemTaskSoli($nrsoliorig, $nrsoliserv, $nrtarefa, $dsrefetarefa)
    {
        $connection = InstanceProvider::getEntityManager()->getConnection();
        $currentCdoperador = $this->environment->getCurrentCdoperador();
        try {
            $connection->beginTransaction();
            $this->basicIntranetFileDir = InstanceProvider::getBasicIntranetFileDir();
            /** @var Entities\Soliserv $soliorig */
            $soliorig = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliorig));
            if (empty($soliorig)) {
                throw new \Exception(Messages::SOLI_INFO);
            }
            /** @var Entities\Soliserv $soliserv */
            $soliserv = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliserv));
            $cdfilialorig = $soliorig->getCdfilial();
            if ($soliserv->getNrsoliorig() != $nrsoliorig) {
                $soliserv->setNrsoliorig($nrsoliorig);
                $this->entityManager->persist($soliserv);
                $this->entityManager->flush();
            }
            $dataQryItemOrigem = InstanceProvider::getRepository()->getLocalSoliserv()->qryItemOrigem($nrsoliorig, $nrsoliserv, $cdfilialorig);
            if (!empty($dataQryItemOrigem)) {
                foreach ($dataQryItemOrigem as $qryItemOrigem) {
                    $stNrSeqItTare = InstanceProvider::getRepository()->getSoliservLocal()->getNrsequittare($nrtarefa, $cdfilialorig);
                    $qryVerifTarefa = InstanceProvider::getRepository()->getLocalSoliserv()->qryVerifTarefa($cdfilialorig, $nrtarefa, $qryItemOrigem["CDITEMTRAB"]);
                    if (empty($qryVerifTarefa)) {
                        $qryInsItemTarefa = InstanceProvider::getFactory()->getItemTarefa()->createItemTarefa(
                            $nrtarefa,
                            $qryItemOrigem["CDITEMTRAB"],
                            $cdfilialorig,
                            $stNrSeqItTare,
                            $dsrefetarefa,
                            $qryItemOrigem["NRPRIOITTARE"],
                            $qryItemOrigem["NRCPLXIMPLEM"],
                            $qryItemOrigem["NRCPLXITEMTB"],
                            $currentCdoperador,
                            null
                        );
                        $this->entityManager->persist($qryInsItemTarefa);
                        $this->entityManager->flush();
                        $dataQryCompAlteItOrig = InstanceProvider::getRepository()->getLocalSoliserv()->qryCompAlteItOrig($qryItemOrigem["CDFILIAL"], $qryItemOrigem["NRTAREFA"], $qryItemOrigem["NRSEQUITTARE"]);
                        foreach ($dataQryCompAlteItOrig as $qryCompAlteItOrig) {
                            $qryInsCompAlt = InstanceProvider::getFactory()->getCompaltittar()->createCompaltittar(
                                $stNrSeqItTare,
                                $cdfilialorig,
                                $nrtarefa,
                                $qryCompAlteItOrig["CDTIPOITTRAB"],
                                $qryCompAlteItOrig["CDCOMPTPITTR"],
                                $qryCompAlteItOrig["QTCOMPALITTF"],
                                $qryCompAlteItOrig["IDRESINFCPAL"]
                            );
                            $this->entityManager->persist($qryInsCompAlt);
                            $this->entityManager->flush();
                        }
                    }
                    $this->copiaRequisito($qryItemOrigem["NRSOLIORIG"], $nrsoliserv, $qryItemOrigem["CDITEMTRAB"]);
                }
            }
            $connection->commit();
        } catch (\Exception $e) {
            try {
                $connection->rollBack();
            } catch (\Exception $e2) {
            }
            throw $e;
        }
    }

    public function copiaRequisito($stSoliOrig, $stSoliServ, $stItemTrab)
    {
        $stDirSoli = $this->basicIntranetFileDir;
        if (empty($stDirSoli)) {
            throw new \Exception(Messages::DIRETORIO_DELPHI);
        }
        $stFileOrigbasic = $stDirSoli . DIRECTORY_SEPARATOR . $stSoliOrig . DIRECTORY_SEPARATOR . 'REQUISITO' . DIRECTORY_SEPARATOR . $stSoliOrig . '_' . $stItemTrab;
        $stFileServbasic = $stDirSoli . DIRECTORY_SEPARATOR . $stSoliServ . DIRECTORY_SEPARATOR . 'REQUISITO' . DIRECTORY_SEPARATOR . $stSoliServ . '_' . $stItemTrab;
        $stFileOrigDoc = $stFileOrigbasic . '.doc';
        $stFileOrigDocx = $stFileOrigbasic . '.docx';
        $stFileServDoc = $stFileServbasic . '.doc';
        $stFileServDocx = $stFileServbasic . '.docx';

        if (file_exists($stFileOrigDoc)) {
            @mkdir("$stDirSoli/$stSoliServ/REQUISITO", 0777, true);
            \Service\Attachment::directoryCheck($stDirSoli . DIRECTORY_SEPARATOR . $stSoliServ);
            if (!copy($stFileOrigDoc, $stFileServDoc)) {
                throw new \Exception(Messages::FAILED_COPY . $stFileOrigDoc);
            }
        }

        if (file_exists($stFileOrigDocx)) {
            @mkdir("$stDirSoli/$stSoliServ/REQUISITO", 0777, true);
            \Service\Attachment::directoryCheck($stDirSoli . DIRECTORY_SEPARATOR . $stSoliServ);
            if (!copy($stFileOrigDocx, $stFileServDocx)) {
                throw new \Exception(Messages::FAILED_COPY . $stFileOrigDocx);
            }
        }
    }

    public function qryCliente()
    {
        $qryCliente = InstanceProvider::getRepository()->getSoliservLocal()->qryCliente();
        foreach ($qryCliente['data'] as &$qryClienteUni) {
            $nrinsjurcliemask = '';
            switch ($qryClienteUni['IDTPIJURCLIE']) {
                case 'J':
                    $nrinsjurcliemask = $this->mask($qryClienteUni['NRINSJURCLIE'], '##.###.###/####-##');
                    break;
                case 'F':
                    $nrinsjurcliemask = $this->mask($qryClienteUni['NRINSJURCLIE'], '###.###.###-##');
                    break;
                case 'O':
                    $nrinsjurcliemask = $this->mask($qryClienteUni['NRINSJURCLIE'], '#####');
                    break;
                case 'I':
                    $nrinsjurcliemask = $qryClienteUni['NRINSJURCLIE'];
                    break;
            }
            $qryClienteUni['NRINSJURCLIEMASK']  = $nrinsjurcliemask;
        }
        return $qryCliente;
    }

    public function mask($val, $mask)
    {
        $maskared = '';
        $k = 0;
        for ($i = 0; $i <= strlen($mask) - 1; $i++) {
            if ($mask[$i] == '#') {
                if (isset($val[$k])) {
                    $maskared .= $val[$k++];
                }
            } else {
                if (isset($mask[$i])) {
                    $maskared .= $mask[$i];
                }
            }
        }
        return $maskared;
    }

    public function validaQryCliente($cdcliente)
    {
        $qryCliente = InstanceProvider::getRepository()->getSoliservLocal()->validaQryCliente($cdcliente, 'TEKNISA%');
        if (!empty($qryCliente["data"])) {
            return 'OK';
        }
    }

    public function qryPesqLicencaUso($cdcliente)
    {
        return InstanceProvider::getRepository()->getSoliservLocal()->qryPesqLicencaUso($cdcliente);
    }

    public function validaqryPesqLicencaUso($cdcliente)
    {
        $qryPesqLicencaUso = InstanceProvider::getRepository()->getSoliservLocal()->validaqryPesqLicencaUso($cdcliente);
        if (!empty($qryPesqLicencaUso["data"])) {
            return 'OK';
        }
    }

    public function qryCodAssunto()
    {
        $qryCodAssunto = InstanceProvider::getRepository()->getSoliservLocal();
        return $qryCodAssunto->qryCodAssunto();
    }

    public function qryVersao($nrsoliserv)
    {
        $soliserv = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliserv));
        $cdprodcomp = $soliserv->getCdprodcomp();
        $cdprodgrup = $soliserv->getCdprodgrup();
        $qryVersao = InstanceProvider::getRepository()->getSoliservLocal();
        return $qryVersao->qryVersao($cdprodcomp, $cdprodgrup);
    }

    public function qryPesqRelease($cdgrupsist, $cdversgis)
    {
        $qryPesqRelease = InstanceProvider::getRepository()->getSoliservLocal();
        return $qryPesqRelease->qryPesqRelease($cdgrupsist, $cdversgis);
    }

    public function qryPatch($cdgrupsist, $cdversgis, $cdreleasever)
    {
        $qryPatch = InstanceProvider::getRepository()->getSoliservLocal();
        return $qryPatch->qryPatch($cdgrupsist, $cdversgis, $cdreleasever);
    }


    public function saveCadAutomatico($referencia, $cdcliente, $nrlicencauso, $cdassunto, $versoes, $nrsoliserv)
    {
        $connection = InstanceProvider::getEntityManager()->getConnection();
        $connection->beginTransaction();
        try {
            $qryParamGeral = InstanceProvider::getRepository()->getSoliservLocal()->qryParamGeral();
            $stOperResp = '';
            $qryFaseTram = InstanceProvider::getRepository()->getSoliservLocal()->qryFaseTram($qryParamGeral[0]["CDFASETRAM"]);
            $qryGrupOperResp = InstanceProvider::getRepository()->getSoliservLocal()->qryGrupOperResp($qryFaseTram[0]["CDGROPERRESP"]);
            $recordCount = count($qryGrupOperResp);
            if ($recordCount < 1) {
                throw new \Exception(Messages::NAO_EXIST_OPER);
            } elseif ($recordCount == 1) {
                $stOperResp = $qryGrupOperResp[0]["CDOPERADOR"];
            } else {
                $stOperResp = $qryGrupOperResp[0]["CDOPERADOR"];
            }
            $soliserv = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliserv));
            $cdfilial = $soliserv->getCdfilial();
            $qryDescSoli = InstanceProvider::getRepository()->getSoliservLocal()->qryDescSoli($cdfilial, $nrsoliserv);
            foreach ($versoes as $versao) {
                $cdrelverprod = !empty($versao["CDRELVERPROD"]) ? $versao["CDRELVERPROD"] : null;
                $cdpatchverpr = !empty($versao["CDPATCHVERPR"])  ? $versao['CDPATCHVERPR'] : null;
                $nrpriosoli   = "000";
                $dsrefesoli   = $soliserv->getDsrefesoli();
                $cdoperador   = $this->environment->getCurrentCdoperador();
                $cdprodcomp   = $soliserv->getCdprodcomp();
                $cdprodgrup   = $versao["CDPRODGRUP"];
                $cdverproorig = $versao["CDVERPRODUTO"];
                $cdfasetram   = $qryParamGeral[0]["CDFASETRAM"];
                $idstatussoli = "P";
                $idsolience   = "N";
                $idsoliorca   = "N";
                $idresdispver = "N";
                $cdopersoli   = $this->environment->getCurrentCdoperador();
                $cdoperresp   = $stOperResp;
                $idsolilida   = "N";
                $cdrelverorig = $cdrelverprod;
                $cdpatrelorig = $cdpatchverpr;
                $dtabersoli   = new \DateTime();
                $qryInsSolicitacao = InstanceProvider::getFactory()->getSoliserv()->createSolicitacao(
                    $cdfilial,
                    $cdrelverorig,
                    $cdpatrelorig,
                    $nrpriosoli,
                    $dsrefesoli,
                    $cdassunto,
                    $cdoperador,
                    $cdprodcomp,
                    $cdprodgrup,
                    $cdverproorig,
                    $cdcliente,
                    $cdfasetram,
                    $idstatussoli,
                    $idsolience,
                    $idsoliorca,
                    $nrlicencauso,
                    $idresdispver,
                    $cdopersoli,
                    $cdoperresp,
                    $idsolilida,
                    $dtabersoli
                );
                $this->entityManager->persist($qryInsSolicitacao);
                $this->entityManager->flush();
                $dtalfasetram =  new \DateTime();
                $dtString = $dtalfasetram->format(\Util\DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                $dsalfasetram = "Referente à solicitação: " . $referencia;
                $txacaolftram = "Abriu a solicitação";
                $nrOrg = $soliserv->getNrorg();
                list($qryInsLogTram, $qryInsLogTramParams) = InstanceProvider::getFactory()->getLogFaseTram()->createLogFaseTramInsertionQuery(
                    $nrOrg,
                    $cdfilial,
                    $qryInsSolicitacao->getNrsoliserv(),
                    $dtString,
                    $cdfasetram,
                    $cdoperador,
                    $cdoperresp,
                    $txacaolftram,
                    $dsalfasetram,
                    $cdoperador
                );
                $connection->executeQuery($qryInsLogTram, $qryInsLogTramParams);
                $description = Utilities::rtf2textPlain($qryDescSoli[0]["DSSOLISERV"], true);
                InstanceProvider::getFactory()->getSoliserv()->insertDescsoli($connection, $qryInsSolicitacao->getNrsoliserv(), $description, $cdfilial);
                $qryInsSolicitacao->setNrorg('20');
                $this->entityManager->persist($qryInsSolicitacao);
                $this->entityManager->flush();
            }
            $connection->commit();
        } catch (\Exception $e) {
            try {
                $connection->rollBack();
            } catch (\Exception $e2) {
            }
            throw $e;
        }
    }

    public function qryProjeto($nrsoliserv)
    {
        $soliserv = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliserv));
        $cdfilial = $soliserv->getCdfilial();
        $qryProjeto = InstanceProvider::getRepository()->getSoliservLocal();
        return $qryProjeto->qryProjeto($nrsoliserv, $cdfilial);
    }

    public function saveDescriptionActivityDetails($cdfilial, $nrtarefa, $nrplativtare, $description)
    {
        $connection = InstanceProvider::getEntityManager()->getConnection();
        try {
            $connection->beginTransaction();
            $entityManager = InstanceProvider::getEntityManager();
            $factoryBudgetPlots = InstanceProvider::getFactory()->getSoliserv();
            /** @var \Teknisa\Model\Teknisa\Entities\Descplativ $descriptionDescplativ */
            $descriptionDescplativ = InstanceProvider::getRepository()->getDescplativ()->findOneBy(array('nrtarefa' => $nrtarefa, 'cdfilial' => $cdfilial, 'nrplativtare' => $nrplativtare));
            if (empty($description)) {
                if (!empty($descriptionDescplativ)) {
                    $entityManager->remove($descriptionDescplativ);
                    $entityManager->flush();
                }
            } else {
                if (empty($descriptionDescplativ)) {
                    $descriptionDescplativ = $factoryBudgetPlots->createDescplativ($cdfilial, $nrtarefa, $nrplativtare);
                    $entityManager->persist($descriptionDescplativ);
                    $entityManager->flush();
                }
                $factoryBudgetPlots->updateTxrfativtare($connection, $cdfilial, $nrtarefa, $nrplativtare, $description);
            }


            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function qryAgrupaSoli($nrsoliserv)
    {
        $soliserv = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliserv));
        $cdfilial = $soliserv->getCdfilial();
        $qryPesqNrSeqAgrupaSoli = InstanceProvider::getRepository()->getSoliservLocal()->qryPesqNrSeqAgrupaSoli($nrsoliserv, $cdfilial);
        $nrseqagrupasoli = null;
        $cdfilialAgrupaSoli = $cdfilial;
        if (!empty($qryPesqNrSeqAgrupaSoli)) {
            $nrseqagrupasoli = $qryPesqNrSeqAgrupaSoli[0]['NRSEQAGRUPASOLI'];
            $cdfilialAgrupaSoli = $qryPesqNrSeqAgrupaSoli[0]['CDFILIAL'];
        }
        return InstanceProvider::getRepository()->getSoliservLocal()->qryAgrupaSoli($nrseqagrupasoli, $cdfilialAgrupaSoli);
    }

    public function qryAgrupaSoliBeforeInsert($nrsoliserv, $nrsoliservGrid)
    {
        $entityManager = InstanceProvider::getEntityManager();
        $factorySoliserv = InstanceProvider::getFactory()->getSoliserv();
        try {
            $soliserv = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliserv));
            $cdfilial = $soliserv->getCdfilial();
            $qryPesqNrSeqAgrupaSoliAfter = InstanceProvider::getRepository()->getSoliservLocal()->qryPesqNrSeqAgrupaSoli($nrsoliserv, $cdfilial);
            $currentCdoperador = $this->environment->getCurrentCdoperador();
            $qryIdsolience = InstanceProvider::getRepository()->getSoliservLocal()->qryIdsolience($nrsoliservGrid);
            if ($qryIdsolience[0]['IDSOLIENCE'] == 'S') {
                throw new \Exception(Messages::SOLI_IDSOLIENCE);
            }
            if (empty($qryPesqNrSeqAgrupaSoliAfter)) {
                $nrseqagrupasoli = Utilities::getCodeByTableName('AGRUPASOLI9999', $soliserv->getNrorg());
                $dependenciasSoliAfter = $factorySoliserv->createAgrupaSoli($nrseqagrupasoli, $cdfilial, $currentCdoperador, $nrsoliserv, $soliserv->getNrorg());
                $entityManager->persist($dependenciasSoliAfter);
                $entityManager->flush();
            } else {
                $nrseqagrupasoli = $qryPesqNrSeqAgrupaSoliAfter[0]['NRSEQAGRUPASOLI'];
            }
            $qryVerificaSolicitacao = InstanceProvider::getRepository()->getSoliservLocal()->qryVerificaSolicitacao($nrsoliserv, $cdfilial);
            $qryPesqNrSeqAgrupaSoli = InstanceProvider::getRepository()->getSoliservLocal()->qryPesqNrSeqAgrupaSoli($nrsoliservGrid, $cdfilial);
            if (empty($qryVerificaSolicitacao)) {
                if (!empty($qryPesqNrSeqAgrupaSoli)) {
                    $nrseqagrupasoli = $qryPesqNrSeqAgrupaSoli[0]['NRSEQAGRUPASOLI'];
                }
            } else {
                if (empty($qryPesqNrSeqAgrupaSoli)) {
                    $params = array(
                        'NRSOLISERVGRID'    => $nrsoliservGrid,
                        'NRSOLISERV'        => $nrsoliserv,
                        'CDFILIAL'          => $cdfilial,
                        'NRSEQAGRUPASOLI'   => $nrseqagrupasoli,
                        'CDOPERINCL'        => $currentCdoperador,
                    );
                    return $params;
                } else {
                    throw new \Exception(Messages::SOLI_DEP_CAD);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function qryAgrupaSoliInsert($cdfilial, $cdoperincl, $nrseqagrupasoli, $nrsoliserv, $nrsoliservGrid)
    {
        $entityManager = InstanceProvider::getEntityManager();
        $factorySoliserv = InstanceProvider::getFactory()->getSoliserv();
        $soliserv = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliserv));
        $dependenciasSoli = $factorySoliserv->createAgrupaSoli($nrseqagrupasoli, $cdfilial, $cdoperincl, $nrsoliservGrid, $soliserv->getNrorg());
        $entityManager->persist($dependenciasSoli);
        $entityManager->flush();
    }

    public function qryAgrupaSoliDelete($rows, $nrsoliserv)
    {
        $connection = InstanceProvider::getEntityManager()->getConnection();
        try {
            $connection->beginTransaction();
            $entityManager = InstanceProvider::getEntityManager();
            foreach ($rows as $row) {
                /** @var \Teknisa\Model\Teknisa\Entities\AgrupaSoli $dependenciasSoli */
                $dependenciasSoli = InstanceProvider::getRepository()->getAgrupaSoli()->findOneBy(array('nrsoliserv' => $row['NRSOLISERV']));
                if (!empty($dependenciasSoli)) {
                    $entityManager->remove($dependenciasSoli);
                    $entityManager->flush();
                }
            }
            $qryAgrupaSoli = $this->qryAgrupaSoli($nrsoliserv);
            if (sizeof($qryAgrupaSoli['data']) == 1) {
                $dependenciasSoli = InstanceProvider::getRepository()->getAgrupaSoli()->findOneBy(array('nrsoliserv' => $nrsoliserv));
                if (!empty($dependenciasSoli)) {
                    $entityManager->remove($dependenciasSoli);
                    $entityManager->flush();
                }
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
}
