<?php
namespace Util;

use Doctrine\ORM\EntityManager;
use Exception;
use Util\Mail\Core\SmtpMail;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Util\Instance\Service as InstanceService;
use Util\Instance\Repository as InstanceRepository;
use Util\Instance\Factory as InstanceFactory;
use Util\Utilities;

class InstanceProvider {

    public static function getService()
    {
        return InstanceService::getInstance();
    }

    /** @return \Util\Instance\Repository */
    public static function getRepository()
    {
        return InstanceRepository::getInstance();
    }

    public static function getFactory()
    {
        return InstanceFactory::getInstance();
    }

    /** @return \Util\Utilities */
    public static function getEnvironment()
    {
        return InstanceManager::getInstance()->getService('\Util\Utilities');
    }

    /** @return \Util\Log */
    public static function getLog()
    {
        return InstanceManager::getInstance()->getService('\Util\Log');
    }

    /** @return \Doctrine\ORM\EntityManager */
    public static function getEntityManager()
    {
        return InstanceManager::getInstance()->getService('entityManager');
    }

    /** @return SmtpMail */
    public static function getSmtpMail()
    {
        return InstanceManager::getInstance()->getService('\Util\Mail\Core\SmtpMail');
    }

    /** @return \Teknisa\Libs\Controller\Api */
    public static function getLibsControllerApi()
    {
        return InstanceManager::getInstance()->getService('\Teknisa\Libs\Controller\Api');
    }

     /** @return \Teknisa\Libs\Util\NewCode */
    public static function getNewCode() {
        return InstanceManager::getInstance()->getService('newCodeSC');
    }

    /**
     * @return \Teknisa\Libs\Util\Environment
     * @throws Exception
     */
    public static function getLibEnvironment()
    {
        return InstanceManager::getInstance()->getService('libEnvironment');
    }

    /** @return \Teknisa\Libs\Util\Utilities */
    public static function getLibsUtilities()
    {
        return InstanceManager::getInstance()->getService('libUtilities');
    }

    /** @return  \Util\Mail\IssueMail */
    public static function getMailIssueMail()
    {
        return InstanceManager::getInstance()->getService('\Util\Mail\IssueMail');
    }

    /** @return  \Util\Mail\SoliservMail */
    public static function getMailSoliservMail()
    {
        return InstanceManager::getInstance()->getService('\Util\Mail\SoliservMail');
    }

    /** @return  \Zeedhi\Framework\HTTP\Kernel */
    public static function getZeedhiFrameworkHTTPKernel()
    {
        return InstanceManager::getInstance()->getService('kernel');
    }

    /** @return  \Zeedhi\Framework\Remote\FileUploader\Server */
    public static function getFileUploaderServe()
    {
        return InstanceManager::getInstance()->getService('fileUploader');
    }

    /** @return  \Zeedhi\Framework\Session\Session */
    public static function getSession()
    {
        return InstanceManager::getInstance()->getService('session');
    }

    /** @return String */
    public static function getBasicIntranetFileDir()
    {
        return InstanceManager::getInstance()->getParameter('basicIntranetFileDir');
    }

    /** @return \Controller\Task */
    public static function getTaskController()
    {
        return InstanceManager::getInstance()->getService('\Controller\Task');
    }

    /** @return \Util\Utilities */
    public static function getUtilities()
    {
        return InstanceManager::getInstance()->getService('\Util\Utilities');
    }

    /** @return \Zeedhi\Framework\DB\Mongo\Mongo
     * @throws Exception
     */
    public static function getMongoDB(): \Zeedhi\Framework\DB\Mongo\Mongo
    {
        return InstanceManager::getInstance()->getService('mongoDB');
    }
}
