import { Metadata, Http, Config } from '@zeedhi/core';
import { ITabsBeforeChangeEventParam, Modal, ModalService, LoadingService, DialogService, FileInput } from '@zeedhi/common';
import { DialogServiceCustom } from '@/services/DialogServiceCustom';
import { AlertServiceCustom } from '@/services/AlertServiceCustom';
import { InstanceService } from '@/services/InstanceService';
import Messages from '@/services/Messages';
import { IterableComponentRender } from '@zeedhi/teknisa-components-common';
import { FileUpload } from '@/services/FileUpload';
import { Utilities } from '@/services/Utilities';
import { ZdList } from '@zeedhi/vuetify';
import Store from '../../store';

export class DemandManagementClosedController {
	public defaultTimeoutTimer: any;

	public callDescription = '';

	public baseUrl = '';

	public callType = '';

	public callRelationship = '';

	public callID = '';

	public isLoadingDefault = false;

	public showDemandManagementForm = false;

	public showFormEditIssue = false;

	private userAuth: any;

	get notIsVisibleIssueActions() {
		return !this.isVisibleIssueActions;
	}

	public getCallRelationshipClosed() {
		return this.callRelationship;
	}

	private modalAddCall!: Modal;

	private modalForwardCall!: Modal;

	private modalIssueActions!: Modal;

	private modalAddComments!: Modal;

	private modalUploadFile!: Modal;

	// START SHARED VARIABLES ADD/EDIT ---
	private typeIsI = false;

	private typeIsS = false;

	private callIsClosed = null;

	public clientNOTSelected = true;

	public clientNOTSelectedAdd = true;

	public productNOTSelected = true;

	/**
	 * Variaveis e métodos de controle de "disable"
	 * dos selects do form de editar issue
	 */
	public clientEditINotSelected = true;

	public productEditINotSelected = true;

	public screenNameEditINotSelected = true;

	public versionEditINotSelected = true;

	public get disableProductEditI() {
		return this.clientEditINotSelected || this.isCallTypeS;
	}

	public get disableCategoryEditI() {
		return this.productEditINotSelected;
	}

	public get disableVersionEditI() {
		return this.screenNameEditINotSelected;
	}

	public get disableDevVersionEditI() {
		return this.screenNameEditINotSelected;
	}

	/** fim */

	get isCallTypeS() {
		return this.callType === 'S';
	}

    get isVisibleDemandManagementGrid() {
		return !this.showDemandManagementForm;
    }

	public get disableProduct() {
		return this.clientNOTSelected;
	}

	public get disableProductAdd() {
		if (this.clientNOTSelectedAdd) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public get disableCategory() {
		return this.productNOTSelected;
	}

	public productNOTSelectedAdd = true;

	public get disableCategoryAdd() {
		if (this.productNOTSelectedAdd) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public screenNameNOTSelected = true;

	public get disableScreenNameAdd() {
		const productEdit = InstanceService.getInstanceZdSelect('productEdit');
		return !Object.keys(productEdit.selectedData).length;
	}

	public get disableVersionI() {
		if (this.screenNameNOTSelected) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public productTypeNOTSelected = true;

	public get disableVersionS() {
		return this.productTypeNOTSelected;
	}

	public versionNOTSelected = true;

	public get disableRelease() {
		return this.versionNOTSelected;
	}

	public releaseNOTSelected = true;

	public get disablePatch() {
		if (this.releaseNOTSelected) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public get disableUseLicense() {
		return this.clientNOTSelected;
	}
	// END SHARED VARIABLES ADD/EDIT ---

	get showTabIssue() {
		return this.showDemandManagementForm && this.callType === 'I';
	}

	get showTabSolicitation() {
		return this.showDemandManagementForm && this.callType === 'S';
	}

	public async onMountedGeneralCard() {
		await window.PM_PROMISE_REQUEST_USER_AUTH;
		this.baseUrl = Config.env.endPoint;
		this.userAuth = Store.getters['User/getUser'];
	}

	public onDestroyDemandManagementGrid() {
		this.modalAddCall?.destroy();
		this.modalForwardCall?.destroy();
	}

	// START CALL GRID ---
	public publicComment = false;

	public checkForwardCall(params: any) {
		return params.row.FORWARD_CALL_GRID === 'S';
	}

	public isIssue(params: any) {
		return params.row.TYPE === 'I';
	}

	public isIssueHasAction(params: any) {
		return params.row.TYPE === 'I' && params.row.HAS_ACTION === 'Y';
	}

	public async rowClickGridDemandManagement(params: any) {
		LoadingService.show();
		this.callDescription = params.row.REFERENCE;
		this.publicComment = false;
		this.showDemandManagementForm = true;
		this.callRelationship = params.row.CALL_RELATIONSHIP;
		this.callType = params.row.TYPE;
		this.callID = params.row.ID;
		this.callIsClosed = params.row.CLOSED;

		try {
			const callData = await Http.get('call', { params: { CALL_RELATIONSHIP: this.callRelationship, CALL_TYPE: this.callType } });
			LoadingService.hide();
			const tabMainFormDemandManagement = { component: null };
			if (params.row.TYPE === 'I') {
				this.showFormEditIssue = true;
				this.typeIsI = true;
				this.typeIsS = false;
				await this.getSelectsEditI(callData);
				tabMainFormDemandManagement.component = Metadata.getInstance<any>('tabMainFormDemandManagementClosed');
			} else {
				this.showDemandManagementForm = true;
				this.typeIsI = false;
				this.typeIsS = true;
				await this.getSelectsEditS(callData);
				tabMainFormDemandManagement.component = Metadata.getInstance<any>('tabMainFormDemandManagementSClosed');
			}
			this.onCreatedIterableComments(tabMainFormDemandManagement);
		} catch (e) {
			LoadingService.hide();
			DialogServiceCustom.showErrorBackend(e);
		}
	}

	public async getSelectsEditI(callData: any) {
		// Atribuir Valor aos selects
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagementIClosed');
		callFormInstance.value.customerIdEditIClosed = callData.data.data.ID; // I S
		callFormInstance.value.teknisaIdEditIClosed = callData.data.data.CALL_RELATIONSHIP; // I S
		callFormInstance.value.reportedByEditIClosed = callData.data.data.REPORTER_NAME; // I S
		callFormInstance.value.referenceEditIClosed = callData.data.data.REFERENCE; // I S
		callFormInstance.value.categoryEditIClosed = callData.data.data.CLASSIFICATION_NAME; // I S
		callFormInstance.value.labelsEditIClosed = callData.data.data.CALL_LABELS; // I S
		callFormInstance.value.customerEditIClosed = callData.data.data.NMRAZSOCCLIE; // I S
		callFormInstance.value.productEditIClosed = callData.data.data.PRODUCT_NAME; // I S
		callFormInstance.value.screenNameEditIClosed = callData.data.data.MENU_ITEM; // I
		callFormInstance.value.versionEditIClosed = callData.data.data.MENU_VERSION; // I
		callFormInstance.value.devVersionEditIClosed = callData.data.data.MENU_VERSION_DEV; // I

		if (callData.data.data.MENU_VERSION_ID_DEV === null) {
			callFormInstance.value.devVersionEditIClosed = 'Última versão';
		}

		callFormInstance.value.initialAppointmentDateEditIClosed = callData.data.data.APPOINTMENT_DATE_INIT; // I S
		callFormInstance.value.appointmentDateEditIClosed = callData.data.data.APPOINTMENT_DATE; // I S
		callFormInstance.value.descriptionEditIClosed = callData.data.data.DESCRIPTION; // I S
	}

	public async getSelectsEditS(callData: any) {
		// Atribuir Valor aos selects
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagementSClosed');
		callFormInstance.value.customerIdEditClosed = callData.data.data.ID; // I S
		callFormInstance.value.teknisaIdEditClosed = callData.data.data.CALL_RELATIONSHIP; // I S
		callFormInstance.value.reportedByEditClosed = callData.data.data.REPORTER_NAME; // I S
		callFormInstance.value.referenceEditClosed = callData.data.data.REFERENCE; // I S
		callFormInstance.value.categoryEditClosed = callData.data.data.CLASSIFICATION_NAME; // I S
		callFormInstance.value.labelsEditClosed = callData.data.data.CALL_LABELS; // I S
		callFormInstance.value.customerEditClosed = callData.data.data.NMRAZSOCCLIE; // I S
		callFormInstance.value.productEditClosed = callData.data.data.PRODUCT_NAME; // I S

		callFormInstance.value.productTypeEditClosed = callData.data.data.PRODUCT_TYPE_NAME; // I S
		callFormInstance.value.versionEditSClosed = callData.data.data.PRODUCT_VERSION_LABEL; // I S
		callFormInstance.value.releaseEditClosed = callData.data.data.PRODUCT_RELEASE_LABEL; // I S
		callFormInstance.value.patchEditClosed = callData.data.data.PRODUCT_PATCH_LABEL; // I S
		callFormInstance.value.useLicenseEditClosed = callData.data.data.USE_LICENSE_LABEL; // I S

		callFormInstance.value.initialAppointmentDateEditClosed = callData.data.data.APPOINTMENT_DATE_INIT; // I S
		callFormInstance.value.appointmentDateEditClosed = callData.data.data.APPOINTMENT_DATE; // I S
		callFormInstance.value.descriptionEditClosed = callData.data.data.DESCRIPTION; // I S
		callFormInstance.value.descriptionEditClosed = callData.data.data.DESCRIPTION; // I S
		callFormInstance.value.soliOriginEditClosed = callData.data.data.NRSOLIORIG; // I S
		callFormInstance.value.responsibleNameByEditClosed = callData.data.data.NMOPERRESP; // I S
	}

	// END CALL GRID -----

	// START MODAL ADD CALL ----
	public addButtonDemandManagementGrid() {
		this.modalAddCall.show();
	}

	public async onMountedFormAddCall() {
		const reportedByAddField = InstanceService.getInstanceZdSelect('reportedByAdd');
		await reportedByAddField.setValue(this.userAuth.CDOPERADOR, false);
	}

	public resetValidations() {
		const addCallInstance = InstanceService.getInstanceZdForm('formAddCall');
		addCallInstance.resetValidation();
	}

	// END MODAL ADD CALL ----

	// START MODAL FORWARD CALL ---
	public currentCustomerForwardCall = '';

	public questionLabelForwardCall = '';

	public disableForwardTo = true;

	public lastCommentLabel = '';

	public forwardCallType = '';

	public forwardCallRelationship = '';

	public REPORTED_BY = '';

	public forwardCDFASTRAMDES = '';

	public forwardNRFASTRAMDES = '';

	public forwardTXACAOFATRAM = '';

	public get forwardToDisable() {
		if (this.disableForwardTo) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public async closeSoli() {
		const sendObject = {
			CDCLIENTE: this.currentCustomerForwardCall,
			CALL_RELATIONSHIP: this.forwardCallRelationship,
		};
		await Http.post('/closeSoliServ', sendObject);

		this.clearModalForwardCall();

		const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
		await callGridInstance.datasource.get();
	}

	public async confirmCloseSoli() {
		const message = Messages.ALERT_CLOSED_CALL_RICH_CLIENT;
		const btnYes = [{
			name: Utilities.getNameDynamic('yes'),
			component: 'ZdButton',
			label: 'Sim',
			outline: false,
			events: {
				click: (() => {
					this.closeSoli();
					DialogService.hide();
				}),
			},
		}];
		DialogServiceCustom.confirmMessage(message, btnYes);
	}

	public async clickOpenForwardCall(params: any) {
		params.event.stopPropagation();
		this.currentCustomerForwardCall = params.row.CDCLIENTE;
		this.forwardCallType = params.row.TYPE;
		this.forwardCallRelationship = params.row.CALL_RELATIONSHIP;
		this.REPORTED_BY = params.row.REPORTED_BY;
		const closeCall = params.row.HAS_ACTION === 'C';
		if (closeCall) {
			await this.confirmCloseSoli();
		} else {
			await this.openModalFornwardSoli();
		}
	}

	private async openModalFornwardSoli() {
		await this.modalForwardCall.show();
		const questionSelectInstance = InstanceService.getInstanceZdSelect('questionSelectForwardCallModal');
		questionSelectInstance.datasource.filter.CALL_TYPE = this.forwardCallType;
		questionSelectInstance.datasource.filter.CALL_RELATIONSHIP = this.forwardCallRelationship;
		questionSelectInstance.datasource.filter.LANG = 'pt';
		try {
			const questionsTest = await questionSelectInstance.datasource.get();
			this.questionLabelForwardCall = `${questionsTest[0].TXPERGFATRAM}*`;
			const sendObject = {
				params: {
					CALL_TYPE: this.forwardCallType,
					CALL_RELATIONSHIP: this.forwardCallRelationship,
				},
			};
			const lastComment = await Http.get('/callComments', sendObject);
			if (lastComment.data.data.length > 0) {
				this.lastCommentLabel = lastComment.data.data[0].DESCRIPTION;
			} else {
				this.lastCommentLabel = 'Não possui cometário!';
			}
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
			this.closeModalForwardCall();
		}
	}

	public async onInputQuestionForwardCall(params: any) {
		const forwardToInstance = InstanceService.getInstanceZdSelect('forwardToForwardCallModal');
		if (params.value) {
			this.resetValidationsForwardCall();
			forwardToInstance.datasource.filter.CDCLIENTE = this.currentCustomerForwardCall;
			forwardToInstance.datasource.filter.CALL_TYPE = this.forwardCallType;
			forwardToInstance.datasource.filter.CALL_RELATIONSHIP = this.forwardCallRelationship;
			forwardToInstance.datasource.filter.CDFASTRAMDES = params.component.selectedData.CDFASTRAMDES;
			forwardToInstance.datasource.filter.NRFASTRAMDES = params.component.selectedData.NRFASTRAMDES;
			try {
				forwardToInstance.value = this.REPORTED_BY;
				const whoFowardResponse = await forwardToInstance.datasource.get();
				if (whoFowardResponse.length === 1) {
					forwardToInstance.value = whoFowardResponse[0].CDOPERADOR;
					this.disableForwardTo = true;
				} else {
					this.disableForwardTo = false;
				}
				this.forwardTXACAOFATRAM = params.component.selectedData.TXACAOFATRAM;
				this.forwardCDFASTRAMDES = params.component.selectedData.CDFASTRAMDES;
				this.forwardNRFASTRAMDES = params.component.selectedData.NRFASTRAMDES;
			} catch (e) {
				DialogServiceCustom.showErrorBackend(e);
				forwardToInstance.value = '';
				this.disableForwardTo = true;
			}
		} else {
			forwardToInstance.value = '';
			this.disableForwardTo = true;
		}
	}

	public validadeCdfastramdes() {
		const aux = !!this.forwardCDFASTRAMDES;
		if (!aux) {
			AlertServiceCustom.customErrorAlert(Messages.ERROR_CDFASE);
		}
		return aux;
	}

	public async saveForwardCall() {
		const forwardCallInstance = InstanceService.getInstanceZdForm('formForwardCallModal');
		const checkValidation = forwardCallInstance.validate();
		if (checkValidation && this.validadeCdfastramdes) {
			const row = {
				CALL_RELATIONSHIP: this.forwardCallRelationship,
				CALL_TYPE: this.forwardCallType,
				CDFASTRAMDES: this.forwardCDFASTRAMDES,
				CDOPERADOR: forwardCallInstance.value.forwardToForwardCallModal,
				COMMENT: forwardCallInstance.value.observationForwardCallModal,
				DOCUMENT: <any>[],
				NRFASTRAMDES: this.forwardNRFASTRAMDES,
				TXACAOFATRAM: this.forwardTXACAOFATRAM,
				requestType: 'DataSet',
			};
			this.isLoadingDefault = true;
			const fileInput: any = Metadata.getInstance('fileInputForwardCallModal');
			if (fileInput.value && fileInput.value.length > 0) {
				let mimeCheck = true;
				const checkFile = fileInput.value.map((file: any) => {
					if (file.size < 50000000) {
						return FileUpload.buildFileSendObject(file);
					}
					mimeCheck = false;
					return false;
				});
				if (mimeCheck) {
					Promise.all(checkFile).then(async (fileArray) => {
						try {
							row.DOCUMENT = fileArray;
							await Http.post('/sendCallGep', row);
							this.closeModalForwardCall();
							AlertServiceCustom.defaultSaveSuccessAlert();
							this.isLoadingDefault = false;
						} catch (e) {
							DialogServiceCustom.showErrorBackend(e);
							this.isLoadingDefault = false;
						}
					});
				} else {
					AlertServiceCustom.customErrorAlert(Messages.ERROR_FILE_TYPE);
					this.isLoadingDefault = false;
				}
			} else {
				try {
					await Http.post('/sendCallGep', row);
					this.closeModalForwardCall();
					AlertServiceCustom.defaultSaveSuccessAlert();
					this.isLoadingDefault = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.isLoadingDefault = false;
				}
			}
        }
	}

	public resetValidationsForwardCall() {
		const forwardCallInstance = InstanceService.getInstanceZdForm('formForwardCallModal');
		forwardCallInstance.resetValidation();
	}

	public closeModalForwardCall() {
		this.modalForwardCall.hide();
		this.clearModalForwardCall();
	}

	public clearModalForwardCall() {
		this.currentCustomerForwardCall = '';
		this.questionLabelForwardCall = '';
		this.disableForwardTo = true;
		this.lastCommentLabel = '';
		this.forwardCallType = '';
		this.forwardCallRelationship = '';
		this.forwardCDFASTRAMDES = '';
		this.forwardNRFASTRAMDES = '';
		this.forwardTXACAOFATRAM = '';
	}
	// END MODAL FORWARD CALL ---

	// START MODAL ISSUE ACTION ---

	private issueActions = [];

	private listIssueActions: any = [];

	private currentIssueAction: any = {};

	private currentIssue: any = {};

	public isVisibleIssueActions = true;

	private getIssueIdByRow(concatenatedIssueId: string) {
		return concatenatedIssueId.replace('I - ', '');
	}

	public async clickIssueActions(params: any) {
		params.event.stopPropagation();
		this.currentIssue = params.row;
		try {
			const issueId = params.row.CALL_RELATIONSHIP;
			LoadingService.show();
			const { data } = await Http.get('/findActionsOfExits', { params: { issueId } });
			if (data.length) {
				await this.openModalIssueActions(data);
			} else {
				DialogServiceCustom.show({
					name: Utilities.getNameDynamic('notIssueActions'),
					title: 'Informação!',
					text: Messages.INFO_NOT_ACTION_ISSUE,
				});
			}
			LoadingService.hide();
		} catch (e) {
			LoadingService.hide();
		}
	}

	public async closeModalIssueActions() {
		this.modalIssueActions.hide();
		this.modalIssueActions.destroy();
		this.listIssueActions = [];
	}

	public async closeIssue() {
		const ISSUE_ID = [this.currentIssue.CALL_RELATIONSHIP];
		const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
		const data = { ISSUE_ID };
		LoadingService.show();
		await Http.post('/closeIssue', data);
		LoadingService.hide();
		AlertServiceCustom.defaultSaveSuccessAlert();
		this.isVisibleIssueActions = true;
		await this.closeModalIssueActions();
		await callGridInstance.datasource.get();
	}

	public clickItemIssueActionsFinalize(param: any) {
		this.currentIssueAction = this.getItemIssueActions(param.component.name);
		this.isVisibleIssueActions = true;
		const message = Messages.ALERT_CLOSED_CALL_RICH_CLIENT;
		const btnYes = [{
			name: Utilities.getNameDynamic('yes'),
			component: 'ZdButton',
			label: 'Sim',
			outline: false,
			events: {
				click: (() => {
					DialogService.hide();
					this.closeIssue();
				}),
			},
		}];
		DialogServiceCustom.confirmMessage(message, btnYes);
	}

	public clickItemIssueActions(param: any) {
		this.currentIssueAction = this.getItemIssueActions(param.component.name);
		this.isVisibleIssueActions = false;

		const formOutCommentModal = this.getFormOutCommentModal();
		formOutCommentModal.value.commentOutCommentModal = this.currentIssueAction.COMMENT_DESCRIPTION;

		const fileInputOutCommentModal = Metadata.getInstance<FileInput>('fileInputOutCommentModal');
		if (this.currentIssueAction.ATTACHMENT_REQUIRED === 'Y') {
			fileInputOutCommentModal.addValidation('required');
		} else {
			fileInputOutCommentModal.removeValidation('required');
		}
	}

	private async openModalIssueActions(data: any) {
		this.modalIssueActions = await ModalService.createFromJson('demandManagement/modal/issueActionsModal', true);
		await this.modalIssueActions.show();
		this.issueActions = data;
		this.createListByData();
		const listIssueActions = Metadata.getInstance<ZdList>('listIssueActions');
		listIssueActions.items = this.listIssueActions;
	}

	private getItemIssueActions(id: any) {
		return this.issueActions.filter((row: any) => row.ID === id)[0];
	}

	private createListByData() {
		this.issueActions.forEach((row: any) => {
			let item = {};
			let events = '{{DemandManagementController.clickItemIssueActions}}';
			if (row.SPECIAL_FUNCTION === 'F') {
				events = '{{DemandManagementController.clickItemIssueActionsFinalize}}';
			}

			item = {
				name: row.ID,
				component: 'ZdListItem',
				title: row.ACTION_LABEL,
				events: {
					click: events,
				},
			};
			this.listIssueActions.push(item);
		});
	}

	// Out Coment

	private getFormOutCommentModal() {
		return InstanceService.getInstanceZdForm('formOutCommentModal');
	}

	public backOutCommentModal() {
		const formOutCommentModal = this.getFormOutCommentModal();
		formOutCommentModal.value.commentOutCommentModal = '';
		formOutCommentModal.value.fileInputOutCommentModal = [];
		formOutCommentModal.resetValidation();
		this.isVisibleIssueActions = true;
	}

	public async saveOutCommentModal() {
		const formOutCommentModal = this.getFormOutCommentModal();
		if (formOutCommentModal.validate()) {
			const fileInput: any = Metadata.getInstance('fileInputOutCommentModal');
			const data: any = { ...this.currentIssueAction, ...formOutCommentModal.value };
			data.DESCRIPTION = data.commentOutCommentModal;
			data.ISSUE_ID = this.currentIssue.CALL_RELATIONSHIP;
			LoadingService.show();
			if (fileInput.value && fileInput.value.length > 0) {
				let mimeCheck = true;

				const checkFile = fileInput.value.map((file: any) => {
					if (file.size < 50000000) {
						return FileUpload.buildFileSendObject(file);
					}
					mimeCheck = false;
					return false;
				});

				if (mimeCheck) {
					Promise.all(checkFile).then(async (fileArray) => {
						try {
							data.DOCUMENT = fileArray;
							await this.sendOutComment(data);
							AlertServiceCustom.defaultSaveSuccessAlert();
						} catch (e) {
							LoadingService.hide();
							DialogServiceCustom.showErrorBackend(e);
						}
					});
				} else {
					AlertServiceCustom.customErrorAlert(Messages.ERROR_FILE_TYPE);
					LoadingService.hide();
				}
			} else {
				try {
					await this.sendOutComment(data);
				} catch (e) {
					LoadingService.hide();
					DialogServiceCustom.showErrorBackend(e);
				}
			}
		}
	}

	private async sendOutComment(data: any) {
		const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
		await Http.post('/sendOutComment', data);
		LoadingService.hide();
		AlertServiceCustom.defaultSaveSuccessAlert();
		this.isVisibleIssueActions = true;
		await this.closeModalIssueActions();
		await callGridInstance.datasource.get();
	}

	// END MODAL ISSUE ACTION ---

	// START CALL TAB ----
	public setIValidationsEDIT() {
		const versionIInstance = InstanceService.getInstanceZdSelect('versionEditI');
		versionIInstance.addValidation('required');
	}

	public removeIValidationsEDIT() {
		const versionIInstance = InstanceService.getInstanceZdSelect('versionEditI');
		versionIInstance.removeValidation('required');
	}

	public setSValidationsEDIT() {
		const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
		const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseEdit');
		const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');
		const releaseInstance = InstanceService.getInstanceZdSelect('releaseEdit');
		const patchInstance = InstanceService.getInstanceZdSelect('patchEdit');
		productTypeInstance.addValidation('required');
		useLicenseInstance.addValidation('required');
		versionSInstance.addValidation('required');
		releaseInstance.addValidation('required');
		patchInstance.addValidation('required');
	}

	public removeSValidationsEDIT() {
		const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
		const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseEdit');
		const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');
		const releaseInstance = InstanceService.getInstanceZdSelect('releaseEdit');
		const patchInstance = InstanceService.getInstanceZdSelect('patchEdit');
		productTypeInstance.removeValidation('required');
		useLicenseInstance.removeValidation('required');
		versionSInstance.removeValidation('required');
		releaseInstance.removeValidation('required');
		patchInstance.removeValidation('required');
	}

	// Ref - I - S
	public resetValidationsEDIT() {
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagement');
		callFormInstance.resetValidation();
	}

	// Ref - I
	public resetInputtedValuesEDIT() { // I S
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagement');
		callFormInstance.value.versionEditI = '';
		callFormInstance.value.productTypeEdit = '';
		callFormInstance.value.versionEditS = '';
		callFormInstance.value.releaseEdit = '';
		callFormInstance.value.patchEdit = '';
		callFormInstance.value.useLicenseEdit = '';
	}

	// Ref - I
	public resetInputtedValuesByProductEditField() {
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagement');
		if (this.typeIsI) {
			callFormInstance.value.screenNameEdit = '';
			callFormInstance.value.versionEditI = '';
			callFormInstance.value.categoryEdit = '';
			callFormInstance.value.devVersionEdit = '';
		}
	}

	// Ref - I - S
	public onChangeProductEditCallModal(params: any) {
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			if (params.value) {
				this.resetInputtedValuesByProductEditField();
				const customerInstance = InstanceService.getInstanceZdSelect('customerEdit');
				const categoryInstance = InstanceService.getInstanceZdSelect('categoryEdit');
				if (params.component.selectValue.PLATAFORMA === 'MOBILE') {
					const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameEdit');
					categoryInstance.datasource.filter.APPLICATION_TYPE = 'I';
					screenNameInstance.datasource.filter.CDCLIENTE = customerInstance.selectedData.CDCLIENTE;
					screenNameInstance.datasource.filter.PRODUCT_ID = params.component.value.PRODUCT_FILTER_ID;
					try {
						await categoryInstance.datasource.get();
						await screenNameInstance.datasource.get();
						this.productNOTSelected = false;
					} catch (e) {
						DialogServiceCustom.showErrorBackend(e);
						this.productNOTSelected = true;
						this.screenNameNOTSelected = true;
						this.productTypeNOTSelected = true;
						this.versionNOTSelected = true;
						this.releaseNOTSelected = true;
					}
				} else {
					const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
					const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseEdit');
					categoryInstance.datasource.filter.APPLICATION_TYPE = 'S';
					productTypeInstance.datasource.filter.PRODUCT_CODE = params.component.datasource.data[0].SGSISTEMA;
					useLicenseInstance.datasource.filter.CDCLIENTE = customerInstance.selectedData.CDCLIENTE;
					try {
						await categoryInstance.datasource.get();
						await productTypeInstance.datasource.get();
						await useLicenseInstance.datasource.get();
						this.productNOTSelected = false;
					} catch (e) {
						DialogServiceCustom.showErrorBackend(e);
						this.productNOTSelected = true;
						this.screenNameNOTSelected = true;
						this.productTypeNOTSelected = true;
						this.versionNOTSelected = true;
						this.releaseNOTSelected = true;
					}
				}
			} else {
				this.productNOTSelected = true;
				this.screenNameNOTSelected = true;
				this.productTypeNOTSelected = true;
				this.versionNOTSelected = true;
				this.releaseNOTSelected = true;
				this.resetInputtedValuesEDIT();
			}
		}, 350);
	}

	// Ref - I
	public onChangeScreenNameEditCallModal(params: any) { // SCREEN NAME EDIT
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			const versionIInstance = InstanceService.getInstanceZdSelect('versionEditI');
			if (params.value) {
				versionIInstance.datasource.filter.PRODUCT_ID = params.component.value.NREXTERNAL_PRODUCT_ID;
				try {
					await versionIInstance.datasource.get();
					this.screenNameNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.screenNameNOTSelected = true;
				}
			} else {
				this.screenNameNOTSelected = true;
				versionIInstance.value = '';
			}
		}, 350);
	}

	// Ref - I
	public async saveEditCall() { // SAVE EDIT
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagement');
		const checkValidation = callFormInstance.validate();
		if (checkValidation) {
			this.isLoadingDefault = true;
			const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
			const customerInstance = InstanceService.getInstanceZdSelect('customerEdit');
			LoadingService.show();
			if (this.typeIsI) {
				const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameEdit');
				try {
					await Http.post('/productCallSave', {
						CALL_RELATIONSHIP: this.callRelationship,
						ID: this.callID,
						REFERENCE: callFormInstance.value.referenceEdit,
						CALL_LABELS: callFormInstance.value.labelsEdit,
						DESCRIPTION: callFormInstance.value.descriptionEdit,
						REPORTED_BY: callFormInstance.value.reportedByEdit,
						CDCLIENTE: customerInstance.selectedData.CDCLIENTE,
						PLATAFORMA: 'MOBILE',
						PROJECT_ID: callFormInstance.value.productEdit.PROJECT_ID,
						SGSISTEMA: null,
						TYPE: 'I',
						PRODUCT_VERSION: '0',
						PRODUCT_RELEASE: '0',
						MENU_ITEM_ID: screenNameInstance.selectedData.ID,
						DEVOPS_ID: screenNameInstance.selectedData.NREXTERNAL_PRODUCT_ID,
						MENU_VERSION_ID: callFormInstance.value.versionEditI,
						VERSION_DEV_ID: callFormInstance.value.devVersionEditI,
						CLASSIFICATION: callFormInstance.value.categoryEdit,
						APPOINTMENT_DATE: callFormInstance.value.appointmentDateEdit,
					});
					AlertServiceCustom.defaultSaveSuccessAlert();
					await callGridInstance.datasource.get();
				} catch (e) {
					LoadingService.hide();
					DialogServiceCustom.showErrorBackend(e);
				}
			} else {
				const productInstance = InstanceService.getInstanceZdSelect('productEdit');
				const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
				const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');
				try {
					await Http.post('/productCallSave', {
						CALL_RELATIONSHIP: this.callRelationship,
						ID: this.callID,
						REFERENCE: callFormInstance.value.referenceEdit,
						PRODUCT_TYPE_NAME: productTypeInstance.selectedData.LABEL,
						MENU_ITEM: '',
						MENU_VERSION: '',
						NMRAZSOCCLIE: customerInstance.selectedData.NMRAZSOCCLIE,
						CALL_LABELS: callFormInstance.value.labelsEdit,
						DESCRIPTION: callFormInstance.value.descriptionEdit,
						REPORTED_BY: callFormInstance.value.reportedByEdit,
						CDCLIENTE: customerInstance.selectedData.CDCLIENTE,
						PLATAFORMA: 'DELPHI',
						PROJECT_ID: callFormInstance.value.productEdit.PROJECT_ID,
						SGSISTEMA: productInstance.selectedData.SGSISTEMA,
						TYPE: 'S',
						PRODUCT: versionSInstance.selectedData.CDPRODCOMP,
						PRODUCT_VERSION: callFormInstance.value.versionEditS,
						PRODUCT_RELEASE: callFormInstance.value.releaseEdit,
						PRODUCT_TYPE: callFormInstance.value.productTypeEdit,
						CDARVPROD: productTypeInstance.selectedData.CDARVPROD,
						CLASSIFICATION: callFormInstance.value.categoryEdit,
						PRODUCT_PATCH: callFormInstance.value.patchEdit,
						USE_LICENSE: callFormInstance.value.useLicenseEdit,
						APPOINTMENT_DATE: callFormInstance.value.appointmentDateEdit,
					});
					AlertServiceCustom.defaultSaveSuccessAlert();
					await callGridInstance.datasource.get();
				} catch (e) {
					LoadingService.hide();
					DialogServiceCustom.showErrorBackend(e);
				}
			}
			this.isLoadingDefault = false;
			LoadingService.hide();
		}
	}

	public clickBackCardFormDemandManagement() {
		this.showDemandManagementForm = false;
		this.showFormEditIssue = false;
		// Call Edit
		this.typeIsI = false;
		this.typeIsS = false;
		this.clientNOTSelected = true;
		this.productNOTSelected = true;
		this.screenNameNOTSelected = true;
		this.productTypeNOTSelected = true;
		this.versionNOTSelected = true;
		this.releaseNOTSelected = true;
		this.resetAllValues();
		// Comment
		const tabsInstance = Metadata.getInstance<ITabsBeforeChangeEventParam>('tabMainFormDemandManagement');
		tabsInstance.activeTab = 0;
	}

	public resetAllValues() {
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagementI');
		callFormInstance.value.customerIdEditI = '';
		callFormInstance.value.teknisaIdEditI = '';
		callFormInstance.value.reportedByEditI = '';
		callFormInstance.value.referenceEditI = '';
		callFormInstance.value.customerEditI = '';
		callFormInstance.value.productEditI = '';
		callFormInstance.value.categoryEditI = '';
		callFormInstance.value.screenNameEditI = '';
		callFormInstance.value.versionEditI = '';
		callFormInstance.value.productTypeEditI = '';
		callFormInstance.value.versionEditS = '';
		callFormInstance.value.releaseEditI = '';
		callFormInstance.value.patchEditI = '';
		callFormInstance.value.useLicenseEditI = '';
		callFormInstance.value.initialAppointmentDateEditI = '';
		callFormInstance.value.appointmentDateEditI = '';
		callFormInstance.value.labelsEditI = '';
		callFormInstance.value.descriptionEditI = '';
	}

	public resetAllValuesFormEditIssue() {
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagementI');
		callFormInstance.value.customerIdEditI = '';
		callFormInstance.value.teknisaIdEditI = '';
		callFormInstance.value.reportedByEditI = '';
		callFormInstance.value.referenceEditI = '';
		callFormInstance.value.customerEditI = '';
		callFormInstance.value.productEditI = '';
		callFormInstance.value.categoryEditI = '';
		callFormInstance.value.screenNameEditI = '';
		callFormInstance.value.versionEditI = '';
		callFormInstance.value.productTypeEditI = '';
		callFormInstance.value.versionEditS = '';
		callFormInstance.value.releaseEditI = '';
		callFormInstance.value.patchEditI = '';
		callFormInstance.value.useLicenseEditI = '';
		callFormInstance.value.initialAppointmentDateEditI = '';
		callFormInstance.value.appointmentDateEditI = '';
		callFormInstance.value.labelsEditI = '';
		callFormInstance.value.descriptionEditI = '';
	}
	// END CALL TAB ------

	// START COMMENTS TAB --
	public onCreatedIterableComments(params: any) {
		const tab = params.component.tabs[params.component.activeTab];
		setTimeout(async () => {
			if (tab.events && tab.events.change && typeof tab.events.change === 'function') {
				await tab.events.change(params);
			}
		}, 100);
		if (params.component.activeTab === 3) {
			setTimeout(async () => {
				await this.updateIterableComments();
			}, 100);
		}
		// DOCUMENTS TAB
		if (params.component.activeTab === 4) {
			setTimeout(async () => {
				let gridAttachments: any = null;

				if (this.callType === 'I') {
					gridAttachments = Metadata.getInstance<any>('gridAttachmentsIClosed');
				} else {
					gridAttachments = Metadata.getInstance<any>('gridAttachmentsSClosed');
				}

				gridAttachments.datasource.filter.CALL_TYPE = this.callType;
				gridAttachments.datasource.filter.CALL_RELATIONSHIP = this.callRelationship;
				try {
					await gridAttachments.datasource.get();
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
				}
			}, 100);
		}
	}

	public onDestroyButtonAddComments() {
		this.modalAddComments?.destroy();
	}

	public clickIterableComments(params: any) {
		console.log(params);
	}

	public onMountedBuildFileIterableData(params: any) {
		const id = params.component.name.split('-')[1];
		let iterableCommentsInstance: any = null;

		if (this.callType === 'I') {
			iterableCommentsInstance = Metadata.getInstance<IterableComponentRender>('iterableCommentsIClosed');
		} else {
			iterableCommentsInstance = Metadata.getInstance<IterableComponentRender>('iterableCommentsSClosed');
		}

		const rowData = iterableCommentsInstance.datasource.data.find((row: { ID: any; }) => row.ID === id);
		if (rowData) {
			if (rowData.ATTACHMENT.length > 0) {
				params.component.datasource.data = rowData.ATTACHMENT;
			}
		}
	}

	public downloadFileComment(params: any) {
		const fileId = params.element.name.split('-')[1];
		const commentId = params.element.name.split('-')[2];
		const fileInstance = Metadata.getInstance<IterableComponentRender>(`iterableFiles-${commentId}`);
		const fileSelected = fileInstance.datasource.data.find((file) => file.ID === fileId);
		if (fileSelected) {
			if (this.callType === 'I') {
				const downloadUri = `${this.baseUrl}/downLoadAttachment?ID=${fileSelected.ID}`;
				window.open(downloadUri);
			} else {
				const fileName = btoa(fileSelected.FILENAME);
				const downloadUri = `${this.baseUrl}/downLoadAttachmentGEP?FILENAME=${fileName}`;
				window.open(downloadUri);
			}
		}
	}

	public async updateIterableComments() {
		let iterableComments: any = null;

		if (this.callType === 'I') {
			iterableComments = Metadata.getInstance<IterableComponentRender>('iterableCommentsIClosed');
		} else {
			iterableComments = Metadata.getInstance<IterableComponentRender>('iterableCommentsSClosed');
		}

		iterableComments.datasource.filter.CALL_TYPE = this.callType;
		iterableComments.datasource.filter.CALL_RELATIONSHIP = this.callRelationship;
		try {
			await iterableComments.datasource.get();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
	}
	// END COMMENTS TAB ----

	// START COMMENTS MODAL ----
	public defaultPublicCommentValue = 'N';

	public async addCommentButton() {
		this.modalAddComments = await ModalService.createFromJson('demandManagement/modal/addCommentsDemandManagementModal', true);
		this.modalAddComments.show();
		if (this.publicComment) {
			this.defaultPublicCommentValue = 'Y';
		}
	}

	public closeAddCommentsModal() {
		this.modalAddComments.hide();
		this.defaultPublicCommentValue = 'N';
		this.modalAddComments?.destroy();
	}

	public get switchButtonDisabledAddCommentsModal() {
		if (this.publicComment) {
			return true;
		}
		return this.isLoadingDefault;
	}
	// END COMMENTS MODAL ----

	// START DOCUMENTS TAB ---
	public onDestroyGridAttachments() {
		this.modalUploadFile.destroy();
	}

	public async addClickGridAttachments() {
		this.modalUploadFile = await ModalService.createFromJson('demandManagement/modal/uploadFileDemandManagementModal', true);
		this.modalUploadFile.show();
	}

	public downloadFileGrid(params: any) {
		if (this.callType === 'I') {
			const downloadUri = `${this.baseUrl}/downLoadAttachment?ID=${params.row.ID}`;
			window.open(downloadUri);
		} else {
			const fileName = btoa(params.row.FILENAME);
			const downloadUri = `${this.baseUrl}/downLoadAttachmentGEP?FILENAME=${fileName}`;
			window.open(downloadUri);
		}
	}

	public async deleteAttachmentsGrid(params: any) {
		params.event.preventDefault();
		const buttonYes = [
			{
				name: Utilities.getNameDynamic('yes-'),
				component: 'ZdButton',
				label: 'Sim',
				events: {
					click: (() => {
						DialogService.hide();
						this.doDeleteAttachmentsGrid(params);
					}),
				},
			},
		];
		DialogServiceCustom.confirmMessage(Messages.DELETE_DEFAULT, buttonYes);
	}

	public async doDeleteAttachmentsGrid(params: any) {
		LoadingService.show();
		try {
			await Http.post('deleteProductCallAttachments', params.component.selectedRows);
			params.component.selectedRows = [];
			await this.updateFileGrid();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
		LoadingService.hide();
	}

	public async updateFileGrid() {
		let gridAttachments: any = null;

		if (this.callType === 'I') {
			gridAttachments = Metadata.getInstance<any>('gridAttachmentsIClosed');
		} else {
			gridAttachments = Metadata.getInstance<any>('gridAttachmentsSClosed');
		}

		try {
			await gridAttachments.datasource.get();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
	}

	public closeUploadFileModal() {
		this.modalUploadFile.hide();
		this.modalUploadFile.destroy();
	}
	// END DOCUMENTS MODAL -----

	public async eventsChangeTabMainFormDemandManagementSClosed(params: any) {
		const activeTab = params.component.tabs[params.component.activeTab];
		if (activeTab.name === 'tabFormActivitiesDemandManagementClosed') {
			setTimeout(async () => {
				await this.reloadActivitiesDemandManagement(this.callRelationship);
			}, 500);
		}
	}

	public async reloadActivitiesDemandManagement(callRelationship: any) {
		try {
			const gridTaskSClosed = InstanceService.getInstanceTekGrid('gridTaskSClosed');
			const gridItemTaskSClosed = InstanceService.getInstanceTekGrid('gridItemTaskSClosed');
			gridTaskSClosed.datasource.filter.NRSOLISERV = callRelationship;
			gridItemTaskSClosed.datasource.filter.NRSOLISERV = callRelationship;
			gridItemTaskSClosed.datasource.filter.NRTAREFA = null;
			await gridTaskSClosed.datasource.get();
			if (gridTaskSClosed.datasource.data[0]) {
				gridTaskSClosed.datasource.currentRow = gridTaskSClosed.datasource.data[0];
			}
		} catch (e) {
			setTimeout(async () => {
				await this.reloadActivitiesDemandManagement(callRelationship);
			}, 500);
		}
	}

    private modalDescriptionClosed!: Modal;

    public async openModalDescriptionClosed() {
        LoadingService.show();
        this.modalDescriptionClosed = await ModalService.createFromJsonPost('', { jsonPath: 'demandManagement/modal/descriptionCallClosedModal' });
        await this.modalDescriptionClosed.show();
        const formDescriptionDemandClosedManagement = InstanceService.getInstanceZdForm('formDescriptionDemandClosedManagement');
        if (this.callType === 'I') {
            const formDemandManagementIClosed = InstanceService.getInstanceZdForm('formDemandManagementIClosed');
            formDescriptionDemandClosedManagement.value.description = formDemandManagementIClosed.value.descriptionEditIClosed;
            LoadingService.hide();
        } else if (this.callType === 'S') {
            const formDemandManagementSClosed = InstanceService.getInstanceZdForm('formDemandManagementSClosed');
            formDescriptionDemandClosedManagement.value.description = formDemandManagementSClosed.value.descriptionEditClosed;
            LoadingService.hide();
        }
    }

    public closemodalDescriptionClosed() {
        this.modalDescriptionClosed.hide();
        this.modalDescriptionClosed.destroy();
    }

    public saveDescriptionClosed() {
        const formDescriptionDemandClosedManagement = InstanceService.getInstanceZdForm('formDescriptionDemandClosedManagement');
        if (this.callType === 'I') {
            const formDemandManagementIClosed = InstanceService.getInstanceZdForm('formDemandManagementIClosed');
            formDemandManagementIClosed.value.descriptionEditIClosed = formDescriptionDemandClosedManagement.value.description;
            this.closemodalDescriptionClosed();
        } else if (this.callType === 'S') {
            const formDemandManagementSClosed = InstanceService.getInstanceZdForm('formDemandManagementSClosed');
            formDemandManagementSClosed.value.descriptionEditClosed = formDescriptionDemandClosedManagement.value.description;
            this.closemodalDescriptionClosed();
        }
    }
}
